Package: caRamel
Type: Package
Title: Automatic Calibration by Evolutionary Multi Objective Algorithm
Version: 1.5
Date: 2026-05-07
Authors@R: c(
  person("Nicolas", "Le Moine", role = c("aut")),
  person("Celine", "Monteil", role = c("aut")),
  person("Frederic", "Hendrickx", role = c("ctb")),
  person("Fabrice", "Zaoui", role = c("aut", "cre"), email = "fabrice.zaoui@edf.fr"),
  person("Alban", "de Lavenne", role = c("ctb"))
  )
Author: Nicolas Le Moine [aut],
  Celine Monteil [aut],
  Frederic Hendrickx [ctb],
  Fabrice Zaoui [aut, cre],
  Alban de Lavenne [ctb]
Maintainer: Fabrice Zaoui <fabrice.zaoui@edf.fr>
Depends: geometry, parallel
Suggests: markdown, rmarkdown, knitr, testthat
Description: The caRamel optimizer has been developed to meet the requirement for an automatic calibration procedure that delivers a family of parameter sets
     that are optimal with regard to a multi-objective target (Monteil et al. <doi:10.5194/hess-24-3189-2020>).
License: GPL-3 | file LICENSE
URL: https://github.com/fzao/caRamel
BugReports: https://github.com/fzao/caRamel/issues
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: knitr
Repository: CRAN
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-12 09:01:29 UTC; I21149
Date/Publication: 2026-05-12 09:40:06 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-12 10:04:29 UTC; unix
Archs: caRamel.so.dSYM

Package: caRamel
Type: Package
Title: Automatic Calibration by Evolutionary Multi Objective Algorithm
Version: 1.5
Date: 2026-05-07
Authors@R: c(
  person("Nicolas", "Le Moine", role = c("aut")),
  person("Celine", "Monteil", role = c("aut")),
  person("Frederic", "Hendrickx", role = c("ctb")),
  person("Fabrice", "Zaoui", role = c("aut", "cre"), email = "fabrice.zaoui@edf.fr"),
  person("Alban", "de Lavenne", role = c("ctb"))
  )
Author: Nicolas Le Moine [aut],
  Celine Monteil [aut],
  Frederic Hendrickx [ctb],
  Fabrice Zaoui [aut, cre],
  Alban de Lavenne [ctb]
Maintainer: Fabrice Zaoui <fabrice.zaoui@edf.fr>
Depends: geometry, parallel
Suggests: markdown, rmarkdown, knitr, testthat
Description: The caRamel optimizer has been developed to meet the requirement for an automatic calibration procedure that delivers a family of parameter sets
     that are optimal with regard to a multi-objective target (Monteil et al. <doi:10.5194/hess-24-3189-2020>).
License: GPL-3 | file LICENSE
URL: https://github.com/fzao/caRamel
BugReports: https://github.com/fzao/caRamel/issues
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: knitr
Repository: CRAN
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-12 09:01:29 UTC; I21149
Date/Publication: 2026-05-12 09:40:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-12 10:20:31 UTC; unix
Archs: caRamel.so.dSYM

useDynLib(caRamel, .registration=TRUE)
importFrom("graphics", "legend", "par", "plot", "points")
importFrom("stats", "rnorm", "runif", "sd")
importFrom("utils", "setTxtProgressBar", "txtProgressBar", "write.table")
import(geometry)
import(parallel)
export(Cextrap)
export(Cinterp)
export(Crecombination)
export(Cusecovar)
export(Dimprove)
export(boxes)
export(caRamel)
export(decrease_pop)
export(dominate)
export(dominated)
export(downsize)
export(matvcov)
export(newXval)
export(pareto)
export(rselect)
export(val2rank)
export(vol_splx)
export(plot_population)
export(plot_caramel)
export(plot_pareto)

Caramel README
==============

What is Caramel?
----------------

Caramel is a certificate management system that makes it easy to use
client certificates in web applications, mobile applications, embedded use
and other places. It solves the certificate signing and certificate
management headache, while attempting to be easy to deploy, maintain and
use in a secure manner.

Caramel makes it easier (it's never completely easy) to run your own
certificate authority and manage and maintain keys and signing periods.

Caramel focuses on reliably and continuously updating short-lived certificates
where clients (and embedded devices) continue to "phone home" and fetch
updated certificates.  This means that we do not have to provide OCSP and CRL
endpoints to handle compromised certificates, but only have to stop updating
the certificate. This also means that expired certificates should be
considered broken.


How does Caramel work?
----------------------

Caramel is a REST-ful web application that accepts certificate signing
requests (CSR) from anyone, stores them, and possibly returns a signed
certificate.

A client pushes its certificate signing request (CSR) to the public web
application. The web application validates the CSR, and stores it in a
database.

A backend (administration) web application talks to the same database
(preferably from inside an intranet or other secure place), and lets you
view signing requests, and sign them.

The certificates are signed with a *short* lifetime (3 days), and client
scripts are supposed to regularly contact the public web service and
download a freshly signed certificate. Root (Certificate Authority) keys
only live on the "administration" part, and should preferably be kept on a
non-public machine.


What should I take special care about?
--------------------------------------

*No* identity validation is performed by the application, this is left as
an exercise for the reader.

Your signing keys are stored on the administration machine.

If a client doesn't regularly download its public certificate, it will
time out and become "stale", thus preventing future connections.


Example usage
-------------

We install the administration interface and database on an internal
machine (intranet) and put the public application on our software update
service. We use the unique identifier for each client (its machine-ID or
mac-address) as the identifier, and manually match these when initially
signing requests.

Since the CSR doesn't change with time, it is re-signed every 15 days, and
in the client startup sequence, the client will download a refreshed
certificate from the caramel server.

Example implementation of a client (in shell-script, using OpenSSL) is
included. The example includes:

- Generating a key
- Generating a signing request
- Uploading a signing request
- Fetching a valid certificate
- Updating a valid certificate


Security trade-offs
-------------------

We have made a conscious decision to have signing keys living
(unencrypted, for now) on the administration server. This is a usability
trade-off in order to make it possible to smoothly use signing keys.

We set strict limits on what kinds of crypto, strings and other things are
allowed in a CSR.

We are doing our very best to **not** build crypto. OpenSSL is used
wherever possible, and we try to **not** implement our own algorithms for
fear of doing it wrongly.


Maintaining your certificates
-----------------------------

In a server environment with several certificates you usually end up
rotating/refreshing them in a cron job.
Included is a tool called `caramel-refresh.sh` that will parse a config file,
and automatically refresh all certificates matching. This requires the original
CSR and certificate to be in place. See `request-certificate` for a tool to
generate your own certificates.

`caramel-refresh` is intended to be run in a cron job on servers (or clients).
Please make sure you run the job as the correct user, so permissions aren't a
problem afterwards.


Getting Started
---------------
```
cd <directory containing this file>
$venv/bin/python setup.py develop
$venv/bin/caramel_initialize_db development.ini
$venv/bin/caramel_ca development.ini
$venv/bin/pserve development.ini
```

Running Tests
-------------
```
cd <directory containing this file>
$venv/bin/python setup.py develop
$venv/bin/python -m unittest discover
```


Running Tests with Nose
-----------------------

```
cd <directory containing this file>
$venv/bin/python setup.py develop
$venv/bin/pip install nose
nosetests
```


Installing the Commit Hook
--------------------------

We use commit-hooks to run test-cases in a clean virtualenv before each commit.
This is to ensure a certain level of quality and code standards, and to prevent
missing dependencies in setup.py. Running these tests at each commit can be
expensive as it involves going to the network and downloading every package
from scratch. This is not a concern for our development environment, but may be
a problem for others.


Install the hooks with the following commands:
```
cd <directory containing this file>
ln -rs pre-commit-checks .git/hooks/pre-commit
```

Please note the "-r" flag to ln, as it makes sure the relative link keeps the
correct path.

For the pre-commit hook to work, you need to have flake8 available. Either
install flake8 via:

`pip install flake8`

Or point git config hooks.flake8 to the flake8 executable:

`git config hooks.flake8 /path/to/flake8`


Dependencies needed from the operating system
---------------------------------------------

* libffi-devel (on RHEL/CentOS)
* openssl, openssl-devel
* gcc


Making sure you have VirtualEnv
-------------------------------

To use the commit hook you need virtualenv available.
If you do not have virtualenv in your path, please point to it with:

`git config hooks.virtualenv /path/to/virtualenv`

In order to run the python3 interpreter instead of the normal python2
interpreter, you should configure the hook as this:

`git config hooks.virtualenv /usr/bin/virtualenv -p /usr/bin/python3`


License
-------
We have chosen the GNU Affero GPL v3 license for the project. We see no need
for others to keep modification to this software a secret, and we welcome
outside providers. Just because the code is GPLv3, doesn't prevent you from
keeping your keys & certificates private.

For the organizations using this, there should be no additional gain to be
had from keeping the source code secret, and if you think there is any such
gain, please contact us.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)

## ----caRa, eval=F, echo=T-----------------------------------------------------
#  library(caRamel)

## ----pbdMPI, eval=F, echo=T---------------------------------------------------
#  library(pbdMPI)

## ----kursawe, eval=F, echo=T--------------------------------------------------
#  kursawe <- function(i) {
#    k1 <- -10 * exp(-0.2 * sqrt(x[i,1]^2 + x[i,2]^2)) - 10 * exp(-0.2 * sqrt(x[i,2]^2 + x[i,3]^2))
#    k2 <- abs(x[i,1])^0.8 + 5 * sin(x[i,1]^3) + abs(x[i,2])^0.8 + 5 * sin(x[i,2]^3) + abs(x[i,3])^0.8 + 5 * sin(x[i,3]^3)
#    return(c(k1, k2))
#  }

## ----kursawe_variable, eval=F, echo=T-----------------------------------------
#  nvar <- 3 # number of variables
#  bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
#  bounds[, 1] <- -5 * bounds[, 1]
#  bounds[, 2] <- 5 * bounds[, 2]

## ----kursawe_objectives, eval=F, echo=T---------------------------------------
#  nobj <- 2 # number of objectives
#  minmax <- c(FALSE, FALSE) # minimization for both functions

## ----kursawe_param, eval=F, echo=T--------------------------------------------
#  popsize <- 100 # size of the genetic population
#  archsize <- 100 # size of the archive for the Pareto front
#  maxrun <- 1000 # maximum number of calls
#  prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

## ----call_caRa, eval=F, echo=T------------------------------------------------
#  results <-
#    caRamel(nobj,
#            nvar,
#            minmax,
#            bounds,
#            kursawe,
#            popsize,
#            archsize,
#            maxrun,
#            prec,
#            carallel = 1,
#            numcores = 20,
#            graph = FALSE,
#            verbose = FALSE)

## ----mpi_caRa, eval=F, echo=T-------------------------------------------------
#  init()  # MPI functions from the pbdMPI package
#  size <- comm.size()
#  rank <- comm.rank()
#  
#  results <- gather(optres, rank.dest = 0)  # gather all results on the main process
#  
#  if (rank == 0) saveRDS(results, "Results.Rds")  # save all the results on disk
#  
#  finalize()

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)

## ----caRa, eval=F, echo=T-----------------------------------------------------
# library(caRamel)

## ----pbdMPI, eval=F, echo=T---------------------------------------------------
# library(pbdMPI)

## ----kursawe, eval=F, echo=T--------------------------------------------------
# kursawe <- function(i) {
#   k1 <- -10 * exp(-0.2 * sqrt(x[i,1]^2 + x[i,2]^2)) - 10 * exp(-0.2 * sqrt(x[i,2]^2 + x[i,3]^2))
#   k2 <- abs(x[i,1])^0.8 + 5 * sin(x[i,1]^3) + abs(x[i,2])^0.8 + 5 * sin(x[i,2]^3) + abs(x[i,3])^0.8 + 5 * sin(x[i,3]^3)
#   return(c(k1, k2))
# }

## ----kursawe_variable, eval=F, echo=T-----------------------------------------
# nvar <- 3 # number of variables
# bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
# bounds[, 1] <- -5 * bounds[, 1]
# bounds[, 2] <- 5 * bounds[, 2]

## ----kursawe_objectives, eval=F, echo=T---------------------------------------
# nobj <- 2 # number of objectives
# minmax <- c(FALSE, FALSE) # minimization for both functions

## ----kursawe_param, eval=F, echo=T--------------------------------------------
# popsize <- 100 # size of the genetic population
# archsize <- 100 # size of the archive for the Pareto front
# maxrun <- 1000 # maximum number of calls
# prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

## ----call_caRa, eval=F, echo=T------------------------------------------------
# results <-
#   caRamel(nobj,
#           nvar,
#           minmax,
#           bounds,
#           kursawe,
#           popsize,
#           archsize,
#           maxrun,
#           prec,
#           carallel = 1,
#           numcores = 20,
#           graph = FALSE,
#           verbose = FALSE)

## ----mpi_caRa, eval=F, echo=T-------------------------------------------------
# init()  # MPI functions from the pbdMPI package
# size <- comm.size()
# rank <- comm.rank()
# 
# results <- gather(optres, rank.dest = 0)  # gather all results on the main process
# 
# if (rank == 0) saveRDS(results, "Results.Rds")  # save all the results on disk
# 
# finalize()

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----caRa---------------------------------------------------------------------
library(caRamel)

## ----schaffer-----------------------------------------------------------------
schaffer <- function(i) {
  if (x[i,1] <= 1) {
    s1 <- -x[i,1]
  } else if (x[i,1] <= 3) {
    s1 <- x[i,1] - 2
  } else if (x[i,1] <= 4) {
    s1 <- 4 - x[i,1]
  } else {
    s1 <- x[i,1] - 4
  }
  s2 <- (x[i,1] - 5) * (x[i,1] - 5)
  return(c(s1, s2))
}

## ----schaffer_variable--------------------------------------------------------
nvar <- 1 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 10 * bounds[, 2]

## ----schaffer_objectives------------------------------------------------------
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min

## ----schaffer_param-----------------------------------------------------------
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

## ----schaffer_launch, fig.show="hide", results="hide"-------------------------
results <-
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          schaffer,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE) # no parallelism

## ----schaffer_OK--------------------------------------------------------------
print(results$success==TRUE)

## ----schaffer_plot1-----------------------------------------------------------
plot(results$objectives[,1], results$objectives[,2], main="Schaffer Pareto front", xlab="Objective #1", ylab="Objective #2")

## ----schaffer_plot2-----------------------------------------------------------
plot(results$parameters, main="Corresponding values for X", xlab="Element of the archive", ylab="X Variable")

## ----kursawe------------------------------------------------------------------
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
  k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
  return(c(k1, k2))
}

## ----kursawe_variable---------------------------------------------------------
nvar <- 3 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]

## ----kursawe_objectives-------------------------------------------------------
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min

## ----kursawe_param, fig.show="hide", results="hide"---------------------------
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

results <- 
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          kursawe,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE) # no parallelism

## ----kursawe_OK_plot----------------------------------------------------------
print(results$success==TRUE)

plot(results$objectives[,1], results$objectives[,2], main="Kursawe Pareto front", xlab="Objective #1", ylab="Objective #2")

## ----kursawe_plot_conv--------------------------------------------------------
matplot(results$save_crit[,1],cbind(results$save_crit[,2],results$save_crit[,3]),type="l",col=c("blue","red"), main="Convergence", xlab="Number of calls", ylab="Objectives values")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----caRa---------------------------------------------------------------------
library(caRamel)

## ----schaffer-----------------------------------------------------------------
schaffer <- function(i) {
  s1 <- x[i,1] * x[i,1]
  s2 <- (x[i,1] - 2) * (x[i,1] - 2)
  return(c(s1, s2))
}

## ----schaffer_variable--------------------------------------------------------
nvar <- 1 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -10 * bounds[, 1]
bounds[, 2] <- 10 * bounds[, 2]

## ----schaffer_objectives------------------------------------------------------
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min

## ----schaffer_param-----------------------------------------------------------
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

## ----schaffer_launch, fig.show="hide", results="hide"-------------------------
results <-
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          schaffer,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE, 
          sensitivity=TRUE) # sensitivity required

## ----schaffer_OK--------------------------------------------------------------
print(results$success==TRUE)

## ----schaffer_plot1-----------------------------------------------------------
plot(results$objectives[,1], results$objectives[,2], main="Schaffer Pareto front", xlab="Objective #1", ylab="Objective #2")

## ----schaffer_plot2-----------------------------------------------------------
plot(results$parameters, main="Corresponding values for X", xlab="Element of the archive", ylab="X Variable")

## ----jacobian-----------------------------------------------------------------
names(results$derivatives)

## ----sensi--------------------------------------------------------------------
plot(results$parameters, results$derivatives$Jacobian_1, main="Sensitivitiy for the first objective", ylab="Sensitivity values", xlab="X values")

## ----histo--------------------------------------------------------------------
hist(results$derivatives$Jacobian_2, main="Sensitivitiy for the second objective", xlab="Sensitivity values", ylab="Distribution of the Pareto front")

## ----sensi2-------------------------------------------------------------------
plot(results$derivatives$Jacobian_1, results$derivatives$Jacobian_2, main="Sensitivitiy for both objectives", ylab="Sensitivity values #2", xlab="Sensitivity values #1")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)

## ----caRa, eval=F, echo=T-----------------------------------------------------
#  library(caRamel)

## ----reti, eval=F, echo=T-----------------------------------------------------
#  library(reticulate)

## ----load, eval=F, echo=T-----------------------------------------------------
#  use_python("/usr/local/bin/python3")
#  source_python("kursawe.py")

## ----wrap, eval=F, echo=T-----------------------------------------------------
#  wrapperFunction <- function(i) {
#    # load the package
#    library(reticulate)
#    # python path
#    use_python("/usr/local/bin/python3")
#    # source the Python function
#    source_python("kursawe.py")
#    # call the Python function and return the results
#    return(kursawe(x[i,]))
#  }

## ----kursawe_variable, eval=F, echo=T-----------------------------------------
#  nvar <- 3 # number of variables
#  bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
#  bounds[, 1] <- -5 * bounds[, 1]
#  bounds[, 2] <- 5 * bounds[, 2]

## ----kursawe_objectives, eval=F, echo=T---------------------------------------
#  nobj <- 2 # number of objectives
#  minmax <- c(FALSE, FALSE) # min and min

## ----kursawe_param, , eval=F, echo=T, fig.show="hide", results="hide"---------
#  popsize <- 100 # size of the genetic population
#  archsize <- 100 # size of the archive for the Pareto front
#  maxrun <- 1000 # maximum number of calls
#  prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
#  
#  results <-
#    caRamel(nobj,
#            nvar,
#            minmax,
#            bounds,
#            wrapperFunction, # It's the wrapper function that will be called
#            popsize,
#            archsize,
#            maxrun,
#            prec)

## ----kursawe_OK_plot, eval=F, echo=T------------------------------------------
#  print(results$success==TRUE)
#  
#  plot(results$objectives[,1], results$objectives[,2], main="Kursawe Pareto front", xlab="Objective #1", ylab="Objective #2")

## ----kursawe_plot_conv, eval=F, echo=T----------------------------------------
#  matplot(results$save_crit[,1],cbind(results$save_crit[,2],results$save_crit[,3]),type="l",col=c("blue","red"), main="Convergence", xlab="Number of calls", ylab="Objectives values")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----caRa---------------------------------------------------------------------
library(caRamel)

## ----kursawe------------------------------------------------------------------
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
  k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
  return(c(k1, k2))
}

## ----parameters---------------------------------------------------------------
nvar <- 3  # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]
nobj <- 2  # number of objectives
minmax <- c(FALSE, FALSE) # minimization for both objectives
popsize <- 100  # size of the genetic population
archsize <- 100  # size of the archive for the Pareto front
maxrun <- 1000  # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj)  # convergence criteria

## ----multi--------------------------------------------------------------------
nrepeat <- 3  # number of calls to caRamel
concat_results_objectives <- NULL  # save results for all the calls
concat_results_parameters <- NULL

for (i in seq(nrepeat)) {
   optres <- caRamel(nobj,
               nvar,
               minmax,
               bounds,
               kursawe,
               popsize,
               archsize,
               maxrun,
               prec,
               carallel = 0,
               graph = FALSE,
               verbose = FALSE)
   
   concat_results_objectives <- rbind(concat_results_objectives,
                                      optres$objectives)
   concat_results_parameters <- rbind(concat_results_parameters,
                                      optres$parameters)
}

## ----merge--------------------------------------------------------------------
results_objectives <- concat_results_objectives
results_objectives[, !minmax] <- -results_objectives[, !minmax]  # important !

is_pareto <- pareto(results_objectives)  # mask

global_results_objectives <- concat_results_objectives[is_pareto, ]  # front from the three previous fronts
global_results_parameters <- concat_results_parameters[is_pareto, ]

## ----plot---------------------------------------------------------------------
plot(concat_results_objectives[, 1], concat_results_objectives[, 2],
     main = "Kursawe Pareto fronts",
     xlab = "Objective #1", ylab = "Objective #2")
points(global_results_objectives[, 1], global_results_objectives[, 2],
       col = "red", pch = "*")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)

## ----caRa, eval=F, echo=T-----------------------------------------------------
# library(caRamel)

## ----reti, eval=F, echo=T-----------------------------------------------------
# library(reticulate)

## ----load, eval=F, echo=T-----------------------------------------------------
# use_python("/usr/local/bin/python3")
# source_python("kursawe.py")

## ----wrap, eval=F, echo=T-----------------------------------------------------
# wrapperFunction <- function(i) {
#   # load the package
#   library(reticulate)
#   # python path
#   use_python("/usr/local/bin/python3")
#   # source the Python function
#   source_python("kursawe.py")
#   # call the Python function and return the results
#   return(kursawe(x[i,]))
# }

## ----kursawe_variable, eval=F, echo=T-----------------------------------------
# nvar <- 3 # number of variables
# bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
# bounds[, 1] <- -5 * bounds[, 1]
# bounds[, 2] <- 5 * bounds[, 2]

## ----kursawe_objectives, eval=F, echo=T---------------------------------------
# nobj <- 2 # number of objectives
# minmax <- c(FALSE, FALSE) # min and min

## ----kursawe_param, , eval=F, echo=T, fig.show="hide", results="hide"---------
# popsize <- 100 # size of the genetic population
# archsize <- 100 # size of the archive for the Pareto front
# maxrun <- 1000 # maximum number of calls
# prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
# 
# results <-
#   caRamel(nobj,
#           nvar,
#           minmax,
#           bounds,
#           wrapperFunction, # It's the wrapper function that will be called
#           popsize,
#           archsize,
#           maxrun,
#           prec)

## ----kursawe_OK_plot, eval=F, echo=T------------------------------------------
# print(results$success==TRUE)
# 
# plot(results$objectives[,1], results$objectives[,2], main="Kursawe Pareto front", xlab="Objective #1", ylab="Objective #2")

## ----kursawe_plot_conv, eval=F, echo=T----------------------------------------
# matplot(results$save_crit[,1],cbind(results$save_crit[,2],results$save_crit[,3]),type="l",col=c("blue","red"), main="Convergence", xlab="Number of calls", ylab="Objectives values")

#' New parameter vectors generation respecting a covariance structure
#'
#' proposes new parameter vectors respecting a covariance structure
#'
#' @param xref : matrix [ . , NPar ] of the reference population whose covariance structure is to be used
#' @param amplif : amplification factor of the standard deviation on each parameter
#' @param n : number of new vectors to generate
#' @return xnew : matrix [ n , NPar ] of new vectors
#'
#' @examples
#' # Definition of the parameters
#' xref <- matrix(rexp(35), 35, 1)
#' amplif <- 2.
#' n <- 5
#' # Call the function
#' res <- Cusecovar(xref, amplif, n)
#'
#' @author Fabrice Zaoui

Cusecovar <- function(xref, amplif, n) {

  g <- apply(xref, 2, mean) # center of reference population (in the parameter space)

  # Calculation of the variances-covariances matrix of the lines
  rr <- matvcov(xref, g)

  # Selection of parameters of non-zero variance and amplification
  sref <- apply(xref, 2, sd)
  sref[which(sref <= 0)] <- NaN
  pvar <- which(!is.na(sref))
  rr <- rr[pvar, pvar]
  d <- amplif * diag(sref[pvar], length(pvar))

  # Calculation of the Cholesky matrix
  tc <- chol(d %*% rr %*% d)

  # Initialization of new sets at the center of gravity
  xnew <- matrix(g,
                 nrow = n,
                 ncol = length(g),
                 byrow = TRUE)

  # Random draw of noise
  eps <- matrix(rnorm(n * length(pvar)), nrow = n)

  # Superposition
  xnew[, pvar] <- xnew[, pvar] + eps %*% tc

  return(xnew)
}

#' Determination of directions for improvement
#'
#' determines directions for improvement
#'
#' @param o_splx : matrix of objectives of simplexes (nrow = npoints, ncol = nobj)
#' @param f_splx : vector (npoints) of associated Pareto numbers (1 = dominated)
#' @return list of elements "oriedge": oriented edges and "ledge": length
#'
#' @examples
#' # Definition of the parameters
#' o_splx <- matrix(rexp(6), 3, 2)
#' f_splx <- c(1,1,1)
#' # Call the function
#' res <- Dimprove(o_splx, f_splx)
#'
#' @author Fabrice Zaoui

Dimprove <- function(o_splx, f_splx) {
  if1 <- which(f_splx == 1)
  ns <- dim(o_splx)[1]
  nobj <- dim(o_splx)[2]
  oriedge <- NULL
  ledge <- NULL

  for (k in seq_len(length(if1))) {
    is_dominated <- dominated(o_splx[if1[k], ], o_splx)
    ideb <- which(is_dominated == TRUE)
    if (length(ideb) > 0) {
      ak <-
        cbind(matrix(ideb), matrix(if1[k], nrow = length(ideb), ncol = 1))
      dobj <- matrix(o_splx[ak[, 2], ] - o_splx[ak[, 1], ], ncol = nobj)
      lk <- matrix(sqrt(apply(dobj**2, 1, sum)), ncol = 1)
      oriedge <- rbind(oriedge, ak)
      ledge <- rbind(ledge, lk)
    }
  }

  return(list("oriedge" = oriedge, "ledge" = ledge))
}

#' Converting the values of a vector into their rank
#'
#' converts the values of a vector into their rank
#'
#' @param X : vector to treat
#' @param opt : integer which gives the rule to follow in case of tied ranks (repeated values): if opt = 1, one returns the average rank, if opt = 2, one returns the corresponding rank in the series of the unique values, if opt = 3, return the max rank
#' @return R : rank vector
#'
#' @examples
#' # Definition of the parameters
#' X <- matrix(rexp(100), 100, 1)
#' opt <- 3
#' # Call the function
#' res <- val2rank(X, opt)
#'
#' @author Fabrice Zaoui

val2rank <- function(X, opt) {

  Rs <- matrix(0., nrow = length(X), ncol = 1)
  R <- matrix(0., nrow = length(X), ncol = 1)

  xso <- sort(X, index.return = TRUE)
  Xs <- xso$x
  ix <- xso$ix
  d <- c(diff(Xs), 1)
  ideb <- 1
  u <- 0

  for (i in seq_len(length(Xs))) {
    if (d[i] != 0) {
      if (opt == 1) {
        Rs[ideb:i, 1] <- (ideb + i) / 2
      } else if (opt == 2) {
        u <- u + 1
        Rs[ideb:i, 1] <- u
      } else{
        Rs[ideb:i, 1] <- i
      }
      ideb <- i + 1
    }
  }
  R[ix, ] <- Rs
  return(R)
}

module pareto_and_co
    use iso_c_binding
    
    implicit none
    
    private
    public pareto, pareto_2d, pareto_3d, dominate, dominated
    
    contains
    
    subroutine dominate(matobj, nind, nobj, f) bind(C, name="dominate_")
        integer(kind = c_int), intent(in), value                :: nind, nobj
        real(kind = c_double), intent(in), dimension(nind,nobj) :: matobj
        integer(kind = c_int), intent(out), dimension(nind)     :: f
        integer, dimension(nind) :: ionion
        integer, allocatable :: ftmp(:), ipeel(:), ileft(:)
        logical, dimension(nind) :: feq0
        integer :: k, i, n, m
        
        ionion = [(i, i=1, nind)] ! overall onion index
        f = 0                     ! if 0 => need to be peeled
        k = 1                     ! peel counter
        feq0 = f == 0
        do while (any(feq0))
            n = count(feq0)
            allocate(ftmp(n))
            allocate(ileft(n))
            ileft = pack(ionion, feq0) ! the left onion
            call pareto(matobj(ileft,:), n, nobj, ftmp)
            m = sum(ftmp)
            allocate(ipeel(m))
            ipeel = pack(ionion(ileft), ftmp == 1) ! the peel 
            f(ipeel) = k 
            k = k + 1
            feq0 = f == 0
            deallocate(ftmp, ipeel, ileft)
        end do

    end subroutine dominate

    subroutine pareto(X, nind, nobj, Ft) bind(C, name="pareto_")
        integer(kind=c_int), intent(in), value :: nind, nobj
        real(kind=c_double), intent(in), dimension(nind, nobj) :: X
        integer(kind=c_int), intent(out), dimension(nind) :: Ft
    
        integer :: i, k, npf, kobj
        integer, allocatable :: pf(:)        ! indices of the current Pareto points
        real(kind=c_double) :: xi_k, xj_k
        logical :: dominated_i, strictly_better_found, equal_so_far
        logical :: any_strict_in_pf
    
        allocate(pf(nind))
        npf = 0
        Ft = 0
    
        do i = 1, nind
            dominated_i = .false.
            ! Comparison of X(i,:) with each Pareto point already retained pf(1..npf)
            do k = 1, npf
                ! Test: does pf(k) dominate i?
                !   pf(k) dominates i <=> X(pf(k),:) >= X(i,:) on all objectives
                !                        AND X(pf(k),:) > X(i,:) on at least one.
                ! As we maximize (cf. caRamel), "better" = larger.
                strictly_better_found = .false.
                equal_so_far = .true.
                any_strict_in_pf = .true.
                do kobj = 1, nobj
                    xi_k = X(i, kobj)
                    xj_k = X(pf(k), kobj)
                    if (xj_k < xi_k) then
                        ! pf(k) is worse on this objective => does not dominate i
                        any_strict_in_pf = .false.
                        exit
                    else if (xj_k > xi_k) then
                        strictly_better_found = .true.
                        equal_so_far = .false.
                    end if
                end do
                if (any_strict_in_pf) then
                    ! pf(k) >= i everywhere; dominates if strictly better somewhere
                    ! or if equal everywhere (duplicate: we keep the first one encountered,
                    ! so i is marked as dominated).
                    if (strictly_better_found .or. equal_so_far) then
                        dominated_i = .true.
                        exit
                    end if
                end if
            end do
    
            if (.not. dominated_i) then
                ! i is non-dominated so far. We add it to pf,
                ! but first we remove from pf the points that i now dominates.
                call prune_and_add(X, nind, nobj, i, pf, npf)
            end if
        end do
    
        Ft = 0
        do k = 1, npf
            Ft(pf(k)) = 1
        end do
        deallocate(pf)
    end subroutine pareto
    
    subroutine prune_and_add(X, nind, nobj, i, pf, npf)
        use iso_c_binding
        integer(kind=c_int), intent(in) :: nind, nobj, i
        real(kind=c_double), intent(in) :: X(nind, nobj)
        integer, intent(inout) :: pf(:)
        integer, intent(inout) :: npf
        integer :: k, w, kobj
        real(kind=c_double) :: xi_k, xj_k
        logical :: i_dominates_k, strictly_better, equal_so_far
    
        w = 0
        do k = 1, npf
            ! does i dominate pf(k)?
            i_dominates_k = .true.
            strictly_better = .false.
            equal_so_far = .true.
            do kobj = 1, nobj
                xi_k = X(i, kobj)
                xj_k = X(pf(k), kobj)
                if (xi_k < xj_k) then
                    i_dominates_k = .false.
                    exit
                else if (xi_k > xj_k) then
                    strictly_better = .true.
                    equal_so_far = .false.
                end if
            end do
            if (i_dominates_k .and. strictly_better) then
                ! pf(k) is dominated by i => we remove it (not rewritten into pf)
                cycle
            else
                w = w + 1
                pf(w) = pf(k)
            end if
        end do
        w = w + 1
        pf(w) = i
        npf = w
    end subroutine prune_and_add

    subroutine pareto_2d(X, nind, Ft) bind(C, name="pareto_2d_")
        integer(kind=c_int), intent(in), value :: nind
        real(kind=c_double), intent(in), dimension(nind, 2) :: X
        integer(kind=c_int), intent(out), dimension(nind) :: Ft
    
        integer :: i
        real(kind=c_double) :: f1_prev, f2_max
    
        if (nind <= 0) return
    
        Ft = 0
        ! The first point (f1 max) is necessarily non-dominated.
        Ft(1) = 1
        f1_prev = X(1, 1)
        f2_max  = X(1, 2)
    
        do i = 2, nind
            ! Thanks to the sorting (f1 decreasing, then f2 decreasing):
            !  - X(i,1) <= X(i-1,1)
            !  - if X(i,1) == f1_prev : f1 duplicate, X(i,2) <= f2_max 
            !    so dominated (or equal to an already retained point) => Ft(i)=0
            !  - if X(i,1) <  f1_prev :
            !      non-dominated if and only if X(i,2) > f2_max (strict)
            if (X(i, 1) < f1_prev) then
                if (X(i, 2) > f2_max) then
                    Ft(i) = 1
                    f2_max = X(i, 2)
                end if
                f1_prev = X(i, 1)
            end if
            ! otherwise: f1 equal to the previous one, 
            !      point dominated/equal => Ft(i) remains 0
        end do
    end subroutine pareto_2d

    subroutine pareto_3d(X, nind, Ft) bind(C, name="pareto_3d_")
        !-------------------------------------------------------------
        ! X is assumed lexicographically sorted DESCENDING on
        ! (f1, f2, f3). We maintain a staircase of current Pareto
        ! points, indexed by decreasing f2 (f3 strictly increasing).
        ! Storage: index array 'stair(:)' of size npf.
        !-------------------------------------------------------------
        integer(kind=c_int), intent(in), value :: nind
        real(kind=c_double), intent(in), dimension(nind, 3) :: X
        integer(kind=c_int), intent(out), dimension(nind) :: Ft
    
        integer, allocatable :: stair(:)
        integer :: i, npf, lo, hi, mid, k, j, new_npf
        real(kind=c_double) :: f2p, f3p
    
        Ft = 0
        if (nind <= 0) return
    
        allocate(stair(nind))
    
        ! First point: always non-dominated (largest f1)
        Ft(1) = 1
        stair(1) = 1
        npf = 1
    
        do i = 2, nind
            f2p = X(i, 2)
            f3p = X(i, 3)
    
            !----- Binary search: smallest k such that X(stair(k),2) < f2p -----
            ! stair is sorted by f2 DESCENDING, so we look for the boundary.
            lo = 1
            hi = npf + 1
            do while (lo < hi)
                mid = (lo + hi) / 2
                if (X(stair(mid), 2) < f2p) then
                    hi = mid
                else
                    lo = mid + 1
                end if
            end do
            k = lo ! 1..npf+1 ; indices < k have f2 >= f2p
    
            !----- Dominance test -----
            ! The potential dominator candidate is stair(k-1) (largest f3
            ! among those with f2 >= f2p, since f3 increases 
            ! with index in stair).
            if (k > 1) then
                if (X(stair(k-1), 3) >= f3p) then
                    ! Dominated (weakly) by stair(k-1).
                    ! Since stair(k-1) has f1 >= f1(i) (sorting) and f2 >= f2p
                    ! and f3 >= f3p, and (f1,f2,f3) cannot be strictly equal
                    ! (otherwise i would come after in descending lex order
                    ! and stair(k-1) already retained)
                    ! => i is strictly dominated.
                    cycle
                end if
            end if
    
            !----- i is non-dominated: insert and purge -----
            Ft(i) = 1
    
            ! Purge: all stair(k), stair(k+1), ... with f3 <= f3p
            ! are dominated by i (i has f2 > their f2, and f3 >= their f3).
            ! Since f3 increases with index, this is a contiguous prefix
            ! starting from k. We remove them by revoking Ft.
            j = k
            do while (j <= npf)
                if (X(stair(j), 3) <= f3p) then
                    Ft(stair(j)) = 0
                    j = j + 1
                else
                    exit
                end if
            end do
    
            ! Rebuild the stair array: elements [1..k-1], then i,
            ! then [j..npf] (those not purged).
            ! Done in place via shifting.
            if (j == k) then
                ! No purge: simple insertion at position k (right shift)
                ! Shift stair(k..npf) one slot to the right
                ! (explicit loop to stay in place)
                ! npf becomes npf+1
                new_npf = npf + 1
                ! Shift from end to start to avoid overwriting
                ! Loop from npf down to k:
                block
                    integer :: t
                    do t = npf, k, -1
                        stair(t+1) = stair(t)
                    end do
                end block
                stair(k) = i
                npf = new_npf
            else
                ! Purge (j - k) elements starting at k, then insert i at k.
                ! New size = npf - (j - k) + 1
                ! If j > k+1, shift stair(j..npf) to stair(k+1..)
                ! If j < k+1 impossible here. If j == k+1, it is exactly
                ! a replacement: stair(k) <- i.
                if (j == k + 1) then
                    stair(k) = i
                    ! npf unchanged
                else
                    ! Shift stair(j..npf) to stair(k+1..k+1+(npf-j))
                    block
                        integer :: t, shift
                        shift = j - (k + 1)   ! > 0
                        do t = j, npf
                            stair(t - shift) = stair(t)
                        end do
                    end block
                    stair(k) = i
                    npf = npf - (j - k) + 1
                end if
            end if
        end do
    
        deallocate(stair)
    end subroutine pareto_3d

    subroutine dominated(Xi, X, nind, nobj, is_dominated) bind(C, name="dominated_")
        
        integer(kind=c_int), intent(in), value :: nind, nobj
        real(kind=c_double), intent(in), dimension(nobj) :: Xi
        real(kind=c_double), intent(in), dimension(nind, nobj) :: X
        integer(kind=c_int), intent(out), dimension(nind) :: is_dominated
        logical, dimension(nind) :: is_efficient
        integer :: j
        
        do j = 1, nind
          is_efficient(j) = any(X(j,:) > Xi) .OR. all(X(j,:) == Xi)
        end do
        is_dominated = merge(0, 1, is_efficient)
        
    end subroutine dominated

end module

#' Box numbering for each points individual of the population
#'
#' This function returns a box number for each points individual of the population
#'
#' @param points : matrix of the objectives
#' @param prec : (double, length = nobj) desired accuracy for the objectives (edges of the boxes)
#' @return vector of numbers for the boxes. boxes[i] gives the number of the box containing points[i].
#'
#' @examples
#' # Definition of the parameters
#' points <- matrix(rexp(200), 100, 2)
#' prec <- c(1.e-3, 1.e-3)
#' # Call the function
#' res <- boxes(points, prec)
#'
#' @author Fabrice Zaoui

boxes <- function(points, prec) {

  n <- dim(points)[1]
  d <- dim(points)[2]

  valmin <- apply(points, 2, min)
  val <- matrix(rep(valmin, n), ncol = d, byrow = TRUE)

  the_box <- points - val
  the_prec <- matrix(rep(prec, n), ncol = d, byrow = TRUE)
  the_box <- floor(the_box / the_prec)

  for (i in 1:d) {
    rankval <- val2rank(the_box[, i], 2)
    the_box[, i] <- rankval
  }

  base <- 1 + apply(the_box, 2, max)
  base <- c(1, base[seq_len(length(base)) - 1])
  base <- cumprod(base)
  base <- matrix(rep(base, n), ncol = d, byrow = TRUE)

  return(rowSums(the_box * base))
}

#' Calculation of the variances-covariances matrix on the reference population
#'
#' calculates the variances-covariances matrix on the reference population
#'
#' @param x : population
#' @param g : center of reference population (in the parameter space)
#' @return rr : variances-covariances matrix on the reference population
#'
#' @examples
#' # Definition of the parameters
#' x <- matrix(rexp(30), 30, 1)
#' g <- mean(x)
#' # Call the function
#' res <- matvcov(x, g)
#'
#' @author Fabrice Zaoui

matvcov <- function(x, g) {

  n <- dim(x)[2]
  rr <- matrix(0, nrow = n, ncol = n)

  xred <- matrix(0, nrow = dim(x)[1], ncol = dim(x)[2])
  ss <- apply(x, 2, sd)
  ss[which(ss <= 0)] <- NaN

  for (i in 1:n) {
    xred[, i] <- (x[, i] - g[i]) / ss[i]
  }

  if (n > 1) {
    for (i1 in 1:(n - 1)) {
      for (i2 in (i1 + 1):n) {
        rr[i1, i2] <- mean(xred[, i1] * xred[, i2])
      }
    }
  }

  rr <- rr + t(rr) + diag(1, n)

  return(rr)
}

#'Rows domination of a matrix by a vector
#'
#' indicates which rows of the matrix Y are dominated by the vector (row) x
#'
#' @param x : row vecteur
#' @param Y : matrix
#' @return D : vector of booleans
#'
#' @examples
#' # Definition of the parameters
#' Y <- matrix(rexp(200), 100, 2)
#' x <- Y[1,]
#' # Call the function
#' res <- dominated(x, Y)
#'
#' @author Alban de Lavenne, Fabrice Zaoui

dominated <- function(x, Y) {

  if (!is.double(x)) {storage.mode(x) <- 'double'}
  if (!is.double(Y)) {storage.mode(Y) <- 'double'}
  res <- .Call(c_dominated, x, Y)
  return(res == 1)

}

#' Downsizing of a population to only one individual per box up to a given accuracy
#'
#' reduces the number of individuals in a population to only one individual per box up to a given accuracy
#'
#' @param points : matrix of objectives
#' @param Fo : rank on the front of each point (1: dominates on the Pareto)
#' @param prec : (double, length = nobj) desired accuracy for sorting objectives
#' @return vector indices
#'
#' @examples
#' # Definition of the parameters
#' points <- matrix(rexp(200), 100, 2)
#' prec <- c(1.e-3, 1.e-3)
#' Fo <- sample(1:100, 100)
#' # Call the function
#' res <- downsize(points, Fo, prec)
#'
#' @author Fabrice Zaoui

downsize <- function(points, Fo, prec) {
  the_box <- boxes(points, prec)

  imax1 <- apply(points, 2, which.max)
  imax1 <- sort(unique(imax1))

  # Conservation of a single point per box at most
  u <- sort(unique(the_box))
  indices <- NULL

  for (i in seq_len(length(u))) {
    iu <- which(the_box == u[i])

    # The candidates to represent the box are limited to those of smaller front
    Fmin <- min(Fo[iu])
    iux <- which(Fo[iu] == Fmin)
    iu <- iu[iux]
    iux <- NULL

    # If in "or" there is one (or more) set that maximizes absolutely one of the obj,
    # it is him that one keeps (case where the box contains at least one point of the front)
    if (Fmin == 1) {
      D <-
        matrix(iu, nrow = length(iu), ncol = 1) %*% matrix(1, nrow = 1, ncol =
                                                             length(imax1)) -
        matrix(1, nrow = length(iu), ncol = 1) %*% matrix(imax1, nrow =
                                                            1, ncol = length(imax1))
      iux <- isTRUE(which(D == 0) > 0)
    }

    if (!is.null(iux)) {
      if (iux == TRUE) {
        iu <- iu[1]
      } else {
        # Otherwise, random draw of a number between 1 and length(iu)
        j <- ceiling(length(iu) * runif(1))
        iu <- iu[j]
      }
    } else{
      j <- ceiling(length(iu) * runif(1))
      iu <- iu[j]
    }

    indices <- c(indices, iu)
  }
  return(indices)
}

#' Plotting of a population of objectives and Pareto front
#'
#' Plots graphs the population regarding each couple of objectives and emphasizes the Pareto front
#'
#' @param MatObj : matrix of the objectives [NInd, nobj]
#' @param nobj : number of objectives (optional)
#' @param objnames : vector, length nobj, of names of the objectives (optional)
#' @param maximized : vector of logical, length nobj, TRUE if objective need to be maximized, FALSE if minimized
#'
#' @examples
#' # Definition of the population
#' Pop <- matrix(runif(300), 100, 3)
#'
#' # Definition of objectives to maximize (Obj1, Obj2) and to minimize (Obj3)
#' maximized <- c(TRUE, TRUE, FALSE)
#'
#' # Call the function
#' plot_pareto(MatObj = Pop, maximized = maximized)
#'
#' @author Celine Monteil

plot_pareto <- function(MatObj, nobj = NULL, objnames = NULL, maximized = NULL) {

  if (is.null(nobj)) nobj <- length(MatObj[1, ])
  if (is.null(objnames)) objnames <- paste("Obj", as.character(c(1:nobj)), sep = "")
  if (is.null(maximized)) maximized <- rep(TRUE, nobj)

  MatTest <- MatObj
  MatTest[, !maximized] <- -MatObj[, !maximized]
  is_pareto <- pareto(MatTest)

  # Graphs
  nb_fen <- choose(n = nobj, k = 2)
  if (nb_fen <= 4) {
    par(mfrow = c(ceiling(nobj / 2), 2 - (nb_fen == 1)))
  } else {
    par(mfrow = c(3, ceiling(nb_fen / 3)))
  }
  l <- seq(1, nobj)
  for (i_fig in 1:(nobj - 1)) {
    l_tmp <- l[-i_fig]
    l_tmp <- l_tmp[l_tmp > i_fig]
    for (i_fig2 in l_tmp) {
      plot(MatObj[, i_fig], MatObj[, i_fig2], xlab = objnames[i_fig], ylab = objnames[i_fig2])
      points(MatObj[is_pareto, i_fig], MatObj[is_pareto, i_fig2], col = 2, pch = 16, cex = 1.2)
    }
  }

}

#' Volume of a simplex
#'
#' calculates the volume of a simplex
#'
#' @param S : matrix (d+1) rows * d columns containing the coordinates in d-dim of d + 1 vertices of a simplex
#' @return V : simplex volume
#'
#' @examples
#' # Definition of the parameters
#' S <- matrix(rexp(6), 3, 2)
#' # Call the function
#' res <- vol_splx(S)
#'
#' @author Fabrice Zaoui

vol_splx <- function(S) {

  d <- dim(S)[2]
  A <- S[2:dim(S)[1], ] - matrix(1, nrow = d, ncol = 1) %*% S[1, ]
  V <- (1 / factorial(d)) * abs(det(A))

  return(V)
}

#' Extrapolation along orthogonal directions to the Pareto front in the space of the objectives
#'
#' gives n new candidates by extrapolation along orthogonal directions to the Pareto front in the space of the objectives
#'
#' @param param : matrix [ NPoints , NPar ] of already evaluated parameters
#' @param crit : matrix [ Npoints , NObj ] of associated criteria
#' @param directions : matrix [ NDir, 2 ] the starting and ending points of the candidate vectors
#' @param longu : matrix [ NDir , 1 ] giving the length of each segment thus defined in the OBJ space (measure of the probability of exploring this direction)
#' @param n : number of new vectors to generate
#' @return xnew : matrix [ n , NPar ] of new vectors
#' @return pcrit : matrix [ n , NObj ] estimated positions of new sets in the goal space
#'
#' @examples
#' # Definition of the parameters
#' param <- matrix(rexp(100), 100, 1)
#' crit <- matrix(rexp(200), 100, 2)
#' directions <- matrix(c(1,3,2,7,13,40), nrow = 3, ncol = 2)
#' longu <- runif(3)
#' n <- 5
#' # Call the function
#' res <- Cextrap(param, crit, directions, longu, n)
#'
#' @author Fabrice Zaoui

Cextrap <- function(param, crit, directions, longu, n) {

  ix <- rselect(n, longu)
  nobj <- dim(crit)[2]
  nvar <- dim(param)[2]

  xnew <- matrix(0, nrow = n, ncol = nvar)
  pcrit <- matrix(0, nrow = n, ncol = nobj)

  for (i in 1:n) {
    iar <- ix[i]

    param_a <- as.matrix(param[directions[iar, ], ])
    crit_a <- crit[directions[iar, ], ]

    # Search vector
    u_param <- diff(param_a)
    u_crit <- diff(crit_a)

    boost <- mean(longu) / longu[iar] # The shorter the edge, the more "boost" the search
    lambda <- -log(1 - runif(1))

    xnew[i, ] <- param_a[2, ] + boost * lambda * u_param # Coordinates in the space of the parameters
    pcrit[i, ] <- crit_a[2, ]  + boost * lambda * u_crit  # Coordinates in the space of the criteria
  }
  return(list("xnew" = xnew, "pcrit" = pcrit))
}

#' Plotting of caRamel results
#'
#' Plot graphs of the Pareto front and a graph of optimization evolution
#'
#' @param caramel_results : list resulting from the caRamel() function, with fields $objectives and $save_crit
#' @param nobj : number of objectives (optional)
#' @param objnames : vector of objectives names (optional)
#'
#' @examples
#' # Definition of the test function
#' viennet <- function(i) {
#'   val1 <- 0.5*(x[i,1]*x[i,1]+x[i,2]*x[i,2])+sin(x[i,1]*x[i,1]+x[i,2]*x[i,2])
#'   val2 <- 15+(x[i,1]-x[i,2]+1)*(x[i,1]-x[i,2]+1)/27+(3*x[i,1]-2*x[i,2]+4)*(3*x[i,1]-2*x[i,2]+4)/8
#'   val3 <- 1/(x[i,1]*x[i,1]+x[i,2]*x[i,2]+1) -1.1*exp(-(x[i,1]*x[i,1]+x[i,2]*x[i,2]))
#'   return(c(val1,val2,val3))
#' }
#' nobj <- 3 # Number of objectives
#' nvar <- 2 # Number of variables
#' minmax <- c(FALSE, FALSE, FALSE) # All the objectives are to be minimized
#' bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # Define the bound constraints
#' bounds[, 1] <- -3 * bounds[, 1]
#' bounds[, 2] <- 3 * bounds[, 2]
#'
#' # Caramel optimization
#' results <- caRamel(nobj, nvar, minmax, bounds, viennet, popsize = 100, archsize = 100,
#'           maxrun = 500, prec = matrix(1.e-3, nrow = 1, ncol = nobj), carallel = FALSE)
#'
#' # Plot of results
#' plot_caramel(results)

plot_caramel <- function(caramel_results, nobj = NULL, objnames = NULL) {

  ngen <- length(caramel_results$save_crit[, 1])
  nrun <- caramel_results$save_crit[ngen, 1]

  if (is.null(nobj)) nobj <- length(caramel_results$objectives[1, ])
  if (is.null(objnames)) {
     objnames <- paste("Obj", as.character(c(1:nobj)), sep = "")
  } else {
     nobj <- length(objnames)
  }

  MatObj <- caramel_results$objectives
  MatEvol <- t(caramel_results$save_crit)

  plot_population(MatObj, nobj, ngen, nrun, objnames, MatEvol)

}

#' MAIN FUNCTION: multi-objective optimizer
#'
#' Multi-objective optimizer. It requires to define a multi-objective function (func) to calibrate the model and bounds on the parameters to optimize.
#'
#' The optimizer was originally written for Scilab by Nicolas Le Moine.
#' The algorithm is a hybrid of the MEAS algorithm (Efstratiadis and Koutsoyiannis (2005) <doi:10.13140/RG.2.2.32963.81446>) by using the directional search method based on the simplexes of the objective space
#'     and the epsilon-NGSA-II algorithm with the method of classification of the parameter vectors archiving management by epsilon-dominance (Reed and Devireddy <doi:10.1142/9789812567796_0004>).
#' Reference : "Multi-objective calibration by combination of stochastic and gradient-like parameter generation rules – the caRamel algorithm"
#'             Celine Monteil (EDF), Fabrice Zaoui (EDF), Nicolas Le Moine (UPMC) and Frederic Hendrickx (EDF)
#'             June 2020 Hydrology and Earth System Sciences 24(6):3189-3209
#'             DOI: 10.5194/hess-24-3189-2020
#' Documentation : "Principe de l'optimiseur CaRaMEL et illustration au travers d'exemples de parametres dans le cadre de la modelisation hydrologique conceptuelle"
#'                 Frederic Hendrickx (EDF) and Nicolas Le Moine (UPMC)
#'                 Report EDF H-P73-2014-09038-FR
#'
#' @param nobj : (integer, length = 1) the number of objectives to optimize (nobj >= 2)
#' @param nvar : (integer, length = 1) the number of variables
#' @param minmax : (logical, length = nobj) the objective is either a minimization (FALSE value) or a maximization (TRUE value)
#' @param bounds : (matrix, nrow = nvar, ncol = 2) lower and upper bounds for the variables
#' @param func : (function) the objective function to optimize. Input argument is the number of parameter set (integer) in the x matrix. The function has to return a vector of at least 'nobj' values (Objectives 1 to nobj are used for optimization, values after nobj are recorded for information.).
#' @param popsize : (integer, length = 1) the population size for the genetic algorithm
#' @param archsize : (integer, length = 1) the size of the Pareto front
#' @param maxrun : (integer, length = 1) the max. number of simulations allowed
#' @param prec : (double, length = nobj) the desired accuracy for the optimization of the objectives
#' @param repart_gene : (integer, length = 4) optional, number of new parameter sets for each rule and per generation
#' @param gpp : (integer, length = 1) optional, calling frequency for the rule "Fireworks"
#' @param blocks (optional): groups for parameters
#' @param pop : (matrix, nrow = nset, ncol = nvar or nvar+nobj ) optional, initial population (used to restart an optimization)
#' @param funcinit (function, optional): the initialization function applied on each node of cluster when parallel computation. The arguments are cl and numcores
#' @param objnames (optional): names of the objectives
#' @param listsave (optional): names of the listing files. Default: None (no output). If exists, fields to be defined: "pmt" (file of parameters on the Pareto Front), "obj" (file of corresponding objective values), "evol" (evolution of maximum objectives by generation). Optional field: "totalpop" (total population and corresponding objectives, useful to restart a computation)
#' @param write_gen : (logical, length = 1) optional, if TRUE, save files 'pmt' and 'obj' at each generation (FALSE by default)
#' @param carallel : (integer, length = 1) optional, do parallel computations? (0: sequential, 1:parallel (default) , 2:user-defined choice)
#' @param numcores : (integer, length = 1) optional, the number of cores for the parallel computations (all cores by default)
#' @param graph : (logical, length = 1) optional, plot graphical output at each generation (TRUE by default)
#' @param sensitivity : (logical, length = 1) optional, compute the first order derivatives of the pareto front (FALSE by default)
#' @param verbose : (logical, length = 1) optional, verbosity mode (TRUE by default)
#' @param worklist : optional values to be transmitted to the user's function (not used)
#
##' @return
##' List of seven elements:
##' \describe{
##' \item{success}{return value (logical, length = 1) : TRUE if successfull}
##' \item{parameters}{Pareto front (matrix, nrow = archsize, ncol = nvar)}
##' \item{objectives}{objectives of the Pareto front (matrix, nrow = archsize, ncol = nobj+nadditional)}
##' \item{derivatives}{list of the Jacobian matrices of the Pareto front if the sensitivity parameter is TRUE or NA otherwise}
##' \item{save_crit}{evolution of the optimal objectives}
##' \item{total_pop}{total population (matrix, nrow = popsize+archsize, ncol = nvar+nobj+nadditional)}
##' \item{gpp}{the calling period for the third generation rule (independent sampling with a priori parameters variance)}
##' }
#'
#' @examples
#' # Definition of the test function
#' viennet <- function(i) {
#'   val1 <- 0.5*(x[i,1]*x[i,1]+x[i,2]*x[i,2])+sin(x[i,1]*x[i,1]+x[i,2]*x[i,2])
#'   val2 <- 15+(x[i,1]-x[i,2]+1)*(x[i,1]-x[i,2]+1)/27+(3*x[i,1]-2*x[i,2]+4)*(3*x[i,1]-2*x[i,2]+4)/8
#'   val3 <- 1/(x[i,1]*x[i,1]+x[i,2]*x[i,2]+1) -1.1*exp(-(x[i,1]*x[i,1]+x[i,2]*x[i,2]))
#'   return(c(val1,val2,val3))
#' }
#' # Number of objectives
#' nobj <- 3
#' # Number of variables
#' nvar <- 2
#' # All the objectives are to be minimized
#' minmax <- c(FALSE, FALSE, FALSE)
#' # Define the bound constraints
#' bounds <- matrix(data = 1, nrow = nvar, ncol = 2)
#' bounds[, 1] <- -3 * bounds[, 1]
#' bounds[, 2] <- 3 * bounds[, 2]
#'
#' # Caramel optimization
#' results <-
#'   caRamel(nobj = nobj,
#'           nvar = nvar,
#'           minmax =  minmax,
#'           bounds = bounds,
#'           func = viennet,
#'           popsize = 100,
#'           archsize = 100,
#'           maxrun = 500,
#'           prec = matrix(1.e-3, nrow = 1, ncol = nobj),
#'           carallel = 0)
#'
#' @author Fabrice Zaoui - Celine Monteil

caRamel <-
  function(nobj,
           nvar,
           minmax,
           bounds,
           func,
           popsize,
           archsize,
           maxrun,
           prec,
           repart_gene = c(5, 5, 5, 5),
           gpp = NULL,
           blocks = NULL,
           pop = NULL,
           funcinit = NULL,
           objnames = NULL,
           listsave = NULL,
           write_gen = FALSE,
           carallel = 1,
           numcores = NULL,
           graph = TRUE,
           sensitivity = FALSE,
           verbose = TRUE,
           worklist = NULL) {

    start_time <- Sys.time()

    # Check the input arguments #
    #############################
    if (nobj <= 1) {
      if (verbose) message("the number of objectives must be greater than one!")
      return(list("success" = FALSE, "message" = "the number of objectives must be greater than one!"))
    }
    if (nvar < 1) {
      if (verbose) message("the number of variables must be greater than zero!")
      return(list("success" = FALSE, "message" = "the number of variables must be greater than zero!"))
    }
    if (length(minmax) != nobj) {
      if (verbose) message("the dimension of 'minmax' is incorrect!")
      return(list("success" = FALSE, "message" = "the dimension of 'minmax' is incorrect!"))
    }
    if ((dim(bounds)[1] != nvar) | (dim(bounds)[2] != 2)) {
      if (verbose) message("the dimension of 'bounds' is incorrect!")
      return(list("success" = FALSE, "message" = "the dimension of 'bounds' is incorrect!"))
    }
    if (!"function" %in% class(func)) {
      if (verbose) message("'func' is not an R function!")
      return(list("success" = FALSE, "message" = "'func' is not an R function!"))
    }
    if (popsize < 1) {
      if (verbose) message("'popsize' must be strictly positive!")
      return(list("success" = FALSE, "message" = "'popsize' must be strictly positive!"))
    }
    if (archsize < 1) {
      if (verbose) message("'archsize' must be strictly positive!")
      return(list("success" = FALSE, "message" = "'archsize' must be strictly positive!"))
    }
    if (maxrun < 1) {
      if (verbose) message("'maxrun' must be strictly positive!")
      return(list("success" = FALSE, "message" = "'maxrun' must be strictly positive!"))
    }
    if (length(repart_gene) != 4) {
      if (verbose) message("the dimension of'repart_gene' must be 4!")
      return(list("success" = FALSE, "message" = "the dimension of 'repart_gene' must be 4!"))
    }
    if (sum(repart_gene <= 0) > 0) {
      if (verbose) message("parameter values for each rule of 'repart_gene' must be strictly positive!")
      return(list("success" = FALSE, "message" = "parameter values for each rule of 'repart_gene' must be strictly positive!"))
    }
    if (!is.null(gpp)) {
      if (gpp < 1) {
        if (verbose) message("gpp must be greater than zero!")
        return(list("success" = FALSE, "message" = "gpp be greater than zero!"))
      }
    }
    initialise_calc <- 0
    if (!is.null(funcinit)) {
      if (!"function" %in% class(funcinit)) {
        if (verbose) message("'funcinit' is not an R function!")
        return(list("success" = FALSE, "message" = "'funcinit' is not an R function!"))
      }
      initialise_calc <- 1
    }
    if (is.null(objnames)) objnames <- paste("Obj", as.character(c(1:nobj)), sep = "")
    writefile <- FALSE
    if (!is.null(listsave)) {
      if (!"list" %in% class(listsave)) {
        if (verbose) message("'listsave' is not an R list!")
        return(list("success" = FALSE, "message" = "'listsave' is not an R list!"))
      }
      writefile <- TRUE
      if (is.null(listsave$pmt)) {
        if (verbose) message(" 'listsave$pmt' must be defined!")
        return(list("success" = FALSE, "message" = " 'listsave$pmt' must be defined!"))
      }
      if (is.null(listsave$obj)) {
        if (verbose) message(" 'listsave$obj' must be defined!")
        return(list("success" = FALSE, "message" = " 'listsave$obj' must be defined!"))
      }
      if (is.null(listsave$evol)) {
        if (verbose) message(" 'listsave$evol' must be defined!")
        return(list("success" = FALSE, "message" = " 'listsave$evol' must be defined!"))
      }
      ecrit_total_pop <- 0
      if (!is.null(listsave$totalpop)) {
        ecrit_total_pop <- 1
      }
    }
    if (write_gen == TRUE) {
      if (writefile == FALSE) {
        if (verbose) message(" 'listsave' must be defined to use write_gen!")
        return(list("success" = FALSE, "message" = " 'listsave' must be defined to use write_gen!"))
      }
      listsave$RadPmt <- gsub(pattern = ".txt", replacement = "", listsave$pmt)
      listsave$RadObj <- gsub(pattern = ".txt", replacement = "", listsave$obj)
      if (ecrit_total_pop == 1) listsave$RadPop <- gsub(pattern = ".txt", replacement = "", listsave$totalpop)
    }
    carallel <- as.integer(carallel)
    if (!(carallel %in% c(0, 1, 2))) {
      if (verbose) message("'carallel' value is not 0, 1 or 2")
      return(list("success" = FALSE, "message" = "wrong value for paramemter 'carallel'"))
    }

    # Initializations
    save_crit <<- c()
    sp <- (bounds[, 2] - bounds[, 1]) / (2 * sqrt(3)) # standard deviation
    gsearch <- ceiling((nvar / 10) * nobj / log(nobj + 1)) # independant search every 'gsearch' iteration
    precis <- matrix(data = prec, nrow = nobj) # precision tolerance for each objective
    nrun <- 0 # main loop index
    ngen <- 0 # generation number
    if (is.null(gpp)) gpp <- ceiling(nvar * (nobj + 1) * 4 / sum(repart_gene))
    jacobian <- NA

    # Init the parallel computation
    if (carallel == 1) {
      if (is.null(numcores)) numcores <- detectCores()
      cl <- makeCluster(numcores)
      if (initialise_calc == 1) funcinit(cl, numcores) # Init for "func"
    }

    # Check the type of the initial population
    if (!is.null(pop)) {
      pop <- as.matrix(pop)
      if(length(pop[1, ]) < (nvar + nobj)) {  # If an initial population exists with no corresponding values for the objectives

        # Evaluation of the objectives
        x <<- pop[, 1:nvar]
        if (carallel == 1) {
          newfeval <- NULL
          clusterExport(cl = cl, varlist = c("x", "worklist"), envir = environment())
          res <- parLapply(cl, 1:dim(x)[1], func)
          for (j in 1:dim(x)[1]) {
            newfeval <- rbind(newfeval, as.numeric(res[[j]][1:nobj]))
          }
        } else if (carallel == 0) { #sequential calls
          newfeval <- matrix(data = 0.,
                             nrow = dim(x)[1],
                             ncol = nobj)
          for (i in 1:dim(x)[1]) {
            res <- func(i)
            newfeval[i, ] <- res[1:nobj]
          }
        } else {
          newfeval <- func(pop[, 1:nvar])
        }
        pop <- cbind(pop[, 1:nvar], newfeval)
        nrun <- nrun + length(pop[, 1])
      }
    }

    # Optimization
    if(verbose) {
      message(paste("Beginning of caRamel optimization <--", date()))
      message(paste("Number of variables :", as.character(nvar)))
      message(paste("Number of functions :", as.character(nobj)))
      pb <- txtProgressBar(min = 0, max = 1, initial = 0, title = "caRamel progress :", label = "caRamel progress :" , style = 3)
    }
    while (nrun < maxrun) {
      ngen <- ngen + 1

      # new population : LHS draw
      if (is.null(pop)) {
          x <- matrix(0, nrow = popsize, ncol = nvar)
          for (j in 1:nvar) {
            # stratification + permutation
            u <- (sample(popsize) - 1 + runif(popsize)) / popsize
            # scaling
            x[, j] <- bounds[j, 1] + u * (bounds[j, 2] - bounds[j, 1])
          }
          
          probj <- matrix(data = NaN,
                          nrow = popsize,
                          ncol = nobj)
      } else {
        vamax <- (ngen %% gpp) == 0
        param <- as.matrix(pop[, 1:nvar])
        dim(param) <- c(dim(pop)[1], nvar)
        if (dim(param)[1] < 4) {
          if (carallel == 1) stopCluster(cl)
          if (verbose) {
            close(pb)
            message("Optimization failed")
          }
          return(list("success" = FALSE, "message" = "The number of feasible points is not sufficient! Try to increase the size of the population..."))
        }
        crit <- pop[, (nvar + 1):(nvar + nobj)]
        dim(crit) <- c(dim(pop)[1], nobj)
        Xp <-
          newXval(param,
                  crit,
                  minmax,
                  sp,
                  bounds,
                  repart_gene,
                  blocks,
                  vamax)
        x <- Xp$x
        probj <- Xp$pcrit
      }

      # simulations
      additional_eval <- NULL
      # parallel calls
      if (carallel == 1) {
        newfeval <- NULL
        clusterExport(cl = cl, varlist = c("x", "worklist"), envir = environment())
        res <- parLapply(cl, 1:dim(x)[1], func)
        for (j in 1:dim(x)[1]) {
          newfeval <- rbind(newfeval, as.numeric(res[[j]][1:nobj]))
          additional_eval <- rbind(additional_eval, res[[j]][(nobj+1):length(res[[1]])])
        }
        nadditional <- ncol(additional_eval)
        if (length(res[[1]]) < (nobj + 1)) {
          additional_eval <- NULL
          nadditional <- 0
        }
      } else if (carallel == 0) { #sequential calls
        newfeval <- matrix(data = 0.,
                           nrow = dim(x)[1],
                           ncol = nobj)
        x <<- x
        for (i in 1:dim(x)[1]) {
          res <- func(i)
          newfeval[i, ] <- res[1:nobj]
          additional_eval <- rbind(additional_eval, res[(nobj + 1):length(res)])
        }
        nadditional <- ncol(additional_eval)
        if (length(res) < (nobj + 1)) {
          additional_eval <- NULL
          nadditional <- 0
        }
      } else {
        newfeval <- func(x)
        additional_eval <- NULL
        nadditional <- 0
      }

      # update the number of calls
      nrun <- nrun + dim(x)[1]

      # show progress
      if (verbose) setTxtProgressBar(pb, min(nrun, maxrun) / maxrun)

      # deal with NaN value
      detect_nan <- is.na(newfeval)
      set_ok <- !rowSums(detect_nan)
      if (length(set_ok[set_ok == TRUE]) == 0) {
        if (carallel == 1) stopCluster(cl)
        if (verbose) {
          message("Optimization failed")
          close(pb)
        }
        return(list("success" = FALSE, "message" = "No feasible points! Try to increase the size of the population..."))
      }
      newfeval <- newfeval[set_ok, ]
      dim(newfeval) <- c(sum(set_ok, na.rm = TRUE), nobj)
      x <- x[set_ok, ]
      dim(x) <- c(sum(set_ok, na.rm = TRUE), nvar)
      additional_eval <- additional_eval[set_ok, ]
      probj <- probj[set_ok, ]
      pop1 <- rbind(pop, cbind(x, newfeval, additional_eval))

      # decrease population
      matobj <- pop1[, (nvar + 1):(nvar + nobj)]
      dim(matobj) <- c(dim(pop1)[1], nobj)
      ind <- decrease_pop(matobj, minmax, prec, archsize, popsize)

      # archive
      arch <- matrix(pop1[ind$arch, ], nrow = length(ind$arch), ncol = nobj + nvar + nadditional)

      # population update
      pop <- pop1[c(ind$arch, ind$pop), ]
      dim(pop) <- c(length(c(ind$arch, ind$pop)), nvar + nobj)
      param_arch <- arch[, 1:nvar]
      crit_arch <- matrix(arch[, (nvar + 1):(nvar + nobj)], nrow = length(ind$arch), ncol = nobj)
      if (nadditional > 0) {
        additional_eval <- matrix(arch[, (nvar + nobj + 1):(nvar + nobj + nadditional)], nrow = length(ind$arch), ncol = nadditional)
      }

      # Records on criteria
      a <- c(lapply(c(1:nobj), function(i) max(crit_arch[, i])))
      maxcrit <- as.data.frame(a, col.names = objnames)
      a <- c(lapply(c(1:nobj), function(i) min(crit_arch[, i])))
      mincrit <- as.data.frame(a, col.names = objnames)
      crit <- mincrit
      crit[minmax] <- maxcrit[minmax]
      save_crit <- cbind(save_crit, c(nrun, t(crit)))

      # Graphs
      if (graph == TRUE) plot_population(MatObj = crit_arch, nobj, ngen, nrun, objnames, MatEvol = save_crit, popsize)

      # Online saves
      if (writefile == TRUE) {
        if (write_gen == TRUE) {
          listsave$pmt <- paste(listsave$RadPmt, "_gen", ngen, ".txt", sep = "")
          listsave$obj <- paste(listsave$RadObj, "_gen", ngen, ".txt", sep = "")
          if (ecrit_total_pop == 1) {
            listsave$totalpop <- paste(listsave$RadPop, "_gen", ngen, ".txt", sep = "")
          }
        }

        write.table(param_arch, listsave$pmt, row.names = FALSE, col.names = FALSE)
        write.table(cbind(crit_arch, additional_eval), listsave$obj, row.names = FALSE, col.names = FALSE)
        write.table(t(save_crit), listsave$evol, row.names = FALSE, col.names = FALSE)
        if (ecrit_total_pop == 1) {
          write.table(pop, listsave$totalpop, row.names = FALSE, col.names = FALSE)
        }
      }
    }

    # Close the progress bar
    if (verbose) close(pb)

    # Compute the first order derivatives
    if (sensitivity == TRUE) {
      if (verbose) message("Computing the sensitivity of the Pareto front...")
      dx <- 0.0001
      dxinv <- 1. / dx
      jacobian <- list()

      xopt <- param_arch
      nfront <- dim(crit_arch)[1]
      dim(xopt) <- c(nfront, nvar)
      x <- xopt
      for (k in 1:nobj) {
        nameId <- paste("Jacobian_", toString(k), sep = "")
        jacobian[[nameId]] <- matrix(data = 0., nrow = nfront, ncol = nvar)
        dim(jacobian[[nameId]]) <- c(nfront, nvar)  # even if only one variable it must be a matrix
      }
      for (j in 1:nvar) {
        x[, j] <- x[, j] + dx
        if (carallel == 1) {
          newfeval <- NULL
          clusterExport(cl = cl, varlist = c("x", "worklist"), envir = environment())
          res <- parLapply(cl, 1:nfront, func)
          for (e in 1:nfront) {
            newfeval <- rbind(newfeval, as.numeric(res[[e]][1:nobj]))
          }
        } else if (carallel == 0 | carallel == 2) { #sequential calls or user-defined choice
          newfeval <- matrix(data = 0.,
                             nrow = nfront,
                             ncol = nobj)
          x <<- x
          for (e in 1:nfront) {
            res <- func(e)
            newfeval[e, ] <- res[1:nobj]
          }
        }
        for (k in 1:nobj) {
          jacobian[[k]][, j] <- (newfeval[, k] - crit_arch[, k]) * dxinv
        }
        x[, j] <- xopt[, j]
      }
      nrun <- nrun + dim(x)[1] # update the number of calls
    }

    # Stop the // cluster
    if (carallel == 1) stopCluster(cl)

    end_time <- Sys.time()
    if (verbose) {
      message(paste("Done in", as.character(end_time - start_time), units(end_time - start_time), "-->", date()))
      message(paste("Size of the Pareto front :", as.character(dim(crit_arch)[1])))
      message(paste("Number of calls :", as.character(nrun)))
    }

    return(list(
      "success" = TRUE,
      "parameters" = param_arch,
      "objectives" = cbind(crit_arch, additional_eval),
      "derivatives" = jacobian,
      "save_crit" = t(save_crit),
      "total_pop" = pop,
      "gpp" = gpp
    ))
  }

#
# Test the checks on the parameters when calling caRamel
#

context("errors")

fonseca <- function(i) {
  s2 <- 1 / sqrt(2)
  val1 <- 1 - exp(-(x[i,1] - s2) * (x[i,1] - s2) - (x[i,2] - s2) * (x[i,2] - s2))
  s2 <- 1 / sqrt(2)
  val2 <- 1 - exp(-(x[i,1] + s2) * (x[i,1] + s2) - (x[i,2] + s2) * (x[i,2] + s2))
  return(c(val1, val2))
}

nvar <- 2 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -4 * bounds[, 1]
bounds[, 2] <- 4 * bounds[, 2]
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

test_that("Right number of objectives", {
    # must be greater than one
    nobj <- 1
    results <-
      caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              popsize,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
    nobj <- 2
})

test_that("Right number of variables", {
    # must be strictly positive"
    nvar <- 0
    results <-
      caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              popsize,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
    nvar <- 2
})

test_that("Test the nmuber of goals", {
    # must be equal to nobj
    minmax <- c(TRUE)
    results <- caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              popsize,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
    minmax <- c(FALSE, FALSE)
})

test_that("Test bound arrays", {
    # must be equal to nvar
    fbounds <- matrix(data = 1, nrow = nvar+1, ncol = 2)
    results <- caRamel(nobj,
              nvar,
              minmax,
              fbounds,
              fonseca,
              popsize,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
})

test_that("Test objective function", {
    # must be R function
    results <- caRamel(nobj,
              nvar,
              minmax,
              bounds,
              "kursawe",
              popsize,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
})

test_that("Test size of the genetic population", {
    # must be positive
    results <- caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              0,
              archsize,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
})

test_that("Test size of the archive", {
    # must be positive
    results <- caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              popsize,
              0,
              maxrun,
              prec)
    expect_true(results$success==FALSE)
})

test_that("Test size of the maximum number of runs", {
    # must be strictly positive
    results <- caRamel(nobj,
              nvar,
              minmax,
              bounds,
              fonseca,
              popsize,
              archsize,
              0,
              prec)
    expect_true(results$success==FALSE)
})

#' Indicates which rows are Pareto
#'
#' indicates which rows of the X criterion matrix are Pareto, when objectives need to be maximized
#'
#' @param X : matrix of objectives [NInd * NObj]
#' @return Ft : vector [NInd], TRUE when the set is on the Pareto front.
#'
#' @examples
#' # Definition of the parameters
#' X <- matrix(runif(200), 100, 2)
#' # Call the function
#' is_pareto <- pareto(X)
#'
#' @author Alban de Lavenne, Fabrice Zaoui

pareto <- function(X) {
  if (!is.double(X)) storage.mode(X) <- 'double'
  nobj <- ncol(X)
  
  if (nobj == 2L) {
    ord <- order(X[, 1], X[, 2], decreasing = TRUE)
    res <- .Call(c_pareto_2d, X[ord, , drop = FALSE]) > 0
  } else if (nobj == 3L) {
    ord <- order(X[, 1], X[, 2], X[, 3], decreasing = TRUE)
    res <- .Call(c_pareto_3d, X[ord, , drop = FALSE]) > 0
  } else {
    ord <- do.call(order, c(lapply(seq_len(nobj), function(k) X[, k]),
                            list(decreasing = TRUE)))
    res <- .Call(c_pareto, X[ord, , drop = FALSE]) > 0
  }
  out <- logical(length(res))
  out[ord] <- res
  out
}

#' Decreasing of the population of parameters sets
#'
#' decreases the population of parameters sets
#'
#' @param matobj : matrix of objectives, dimension (ngames, nobj)
#' @param minmax : vector of booleans, of dimension nobj: TRUE if maximization of the objective, FALSE otherwise
#' @param prec : nobj dimension vector: accuracy
#' @param archsize : integer: archive size
#' @param popsize : integer: population size
##' @return
##' A list containing two elements:
##' \describe{
##' \item{ind_arch}{indices of individuals in the updated Pareto front}
##' \item{ind_pop}{indices of individuals in the updated population}
##' }
#'
#' @examples
#' # Definition of the parameters
#' matobj <- matrix(rexp(200), 100, 2)
#' prec <- c(1.e-3, 1.e-3)
#' archsize <- 100
#' minmax <- c(FALSE, FALSE)
#' popsize <- 100
#' # Call the function
#' res <- decrease_pop(matobj, minmax, prec, archsize, popsize)
#'
#' @author Fabrice Zaoui

decrease_pop <- function(matobj, minmax, prec, archsize, popsize) {

  nobj <- dim(matobj)[2]
  ind_pop <- matrix(data = 1:dim(matobj)[1])
  ind_arch <- NULL

  matobj[, !minmax] <- -matobj[, !minmax]

  Fo <- dominate(matobj)

  # Choice of retained points
  indices <- downsize(matobj, Fo, prec)
  pop <- matobj[indices, ]
  dim(pop) <- c(length(indices), nobj)

  ind_pop <- ind_pop[indices, ]

  #////////////////////// Recalculation of fronts ////////////////////////
  Fo <- dominate(pop)

  # Sort by increasing front
  Fs <- sort(Fo, index.return = TRUE)
  pop <- pop[Fs$ix, ]
  dim(pop) <- c(length(Fs$ix), nobj)
  Fo <- Fo[Fs$ix]

  ind_pop <- ind_pop[Fs$ix]

  # Separation "elite" / rest of the population
  arch <- matrix(pop[Fo == 1, ], ncol = nobj)
  pop <- matrix(pop[Fo > 1, ], ncol = nobj)
  ind_arch <- ind_pop[Fo == 1]
  ind_pop <- ind_pop[Fo > 1]

  Fo <- Fo[Fo > 1]

  # Decreasing the population size
  # If the population size exceeds popsize, elimination of the parameter sets with a high Fo
  if (dim(pop)[1] > popsize) {
    Fmax <- Fo[popsize]
    ind_pop1 <- ind_pop[Fo < Fmax]

    n2 <- popsize - length(ind_pop1)

    if (n2 > 0) { # If still possible, selection of n2 points in addition
      ix2 <- which(Fo == Fmax)
      r <- rselect(n2, matrix(1, nrow = length(ix2), ncol = 1))
      ix2 <- ix2[r]
      ind_pop1 <- c(ind_pop1, ind_pop[ix2])
    }
    ind_pop <- ind_pop1
  }

  #  Decreasing the archive size
  if (dim(arch)[1] > archsize) {
    arch_down <- TRUE
    arch_prec <- 2 * prec
    while (arch_down) {
      indices <-
        downsize(arch, matrix(1, nrow = dim(arch)[1], ncol = 1), arch_prec)
      arch_down <- length(indices) > archsize
      arch_prec <- 2 * arch_prec
    }
    ind_arch <- ind_arch[indices]
  }

  return(list("arch" = ind_arch, "pop" = ind_pop))
}

#' Selection of n points
#'
#' performs a selection of n points in facp
#'
#' @param n : number of points to select
#' @param facp : vector of initial points
#' @return ix : ranks of selected points (vector of dimension n)
#'
#' @examples
#' # Definition of the parameters
#' n <- 5
#' facp <- runif(30)
#' # Call the function
#' res <- rselect(n, facp)
#'
#' @author Fabrice Zaoui

rselect <- function(n, facp) {

  if (sum(facp) > 0 & n > 0) {
    facp1 <- facp / sum(facp)
    Fc <- cumsum(facp1)

    ix <- matrix(0, nrow = n, ncol = 1)

    for (i in 1:n) {
      f <- runif(1)
      b <- which(Fc > f)
      ix[i] <- b[1]
    }
  } else{
    ix <- NULL
  }
  return(ix)
}

#' Recombination of the sets of parameters
#'
#' performs a recombination of the sets of parameters
#'
#' @param param : matrix [ . , NPar ] of the population of parameters
#' @param blocks : list of integer vectors: list of variable blocks for recombination
#' @param n : number of new vectors to generate
#' @return xnew : matrix [ n , NPar ] of new vectors
#'
#' @examples
#' # Definition of the parameters
#' param <- matrix(rexp(15), 15, 1)
#' blocks <- NULL
#' n <- 5
#' # Call the function
#' res <- Crecombination(param, blocks, n)
#'
#' @author Fabrice Zaoui

Crecombination <- function(param, blocks, n) {

  npar <- dim(param)[2]
  paramiso <- matrix(TRUE, ncol = npar)
  nblocks <- length(blocks)
  if (nblocks > 0) {
    for (i in 1:nblocks) {
      paramiso[blocks[[i]]] <- FALSE
    }
  }
  ix <- which(paramiso)

  blocks1 <- blocks
  n1 <- nblocks + length(ix)

  for (i in seq_len(length(ix))) {
    blocks1[[(nblocks + i)]] <- c(ix[i])
  }

  xnew <- matrix(0, nrow = n, ncol = npar)
  for (i in 1:n) {
    gamebloc <- rselect(n1, matrix(1, nrow = dim(param)[1], ncol = 1)) # Selection of n1 sets for n1 blocks
    p <- matrix(0, ncol = npar, nrow = 1)
    for (b in 1:n1) {
      p[blocks1[[b]]] <- param[gamebloc[b], blocks1[[b]]]
    }
    xnew[i, ] <- p
  }

  return(xnew)
}

.onAttach <- function(lib, pkg) {
  version <- read.dcf(file.path(lib, pkg, "DESCRIPTION"), "Version")

  if (interactive()) { # > figlet mascaRet
      packageStartupMessage(
"            _____                      _
           |  __ \\                    | |
   ___ __ _| |__) |__ _ _ __ ___   ___| |
  / __/ _` |  _  // _` | '_ ` _ \\ / _ \\ |
 | (_| (_| | | \\ \\ (_| | | | | | |  __/ |
  \\___\\__,_|_|  \\_\\__,_|_| |_| |_|\\___|_|, version ", version)

} else packageStartupMessage("Package 'caRamel' version ", version)
  invisible()
}

#' Interpolation in simplexes of the objective space
#'
#' proposes n new candidates by interpolation in simplexes of the objective space
#'
#' @param param : matrix [ NPoints , NPar ] of already evaluated parameters
#' @param crit : matrix [ Npoints , NObj ] of associated criteria
#' @param simplices : matrix [ NSimp , NObj+1 ] containing all or part of the triangulation of the space of the objectives
#' @param volume : matrix [ NSimp , 1 ] giving the volume of each simplex (measure of the probability of interpolating in this simplex)
#' @param n : number of new vectors to generate
#' @return xnew : matrix [ n , NPar ] of new vectors
#' @return pcrit : matrix [ n , NObj ] estimated positions of new sets in the goal space
#'
#' @examples
#' # Definition of the parameters
#' param <- matrix(rexp(100), 100, 1)
#' crit <- matrix(rexp(200), 100, 2)
#' simplices <- matrix(c(15,2,1,15,22,1,18,15,2,17,13,14), nrow = 4, ncol = 3)
#' volume <- runif(4)
#' n <- 5
#' # Call the function
#' res <- Cinterp(param, crit, simplices, volume, n)
#'
#' @author Fabrice Zaoui

Cinterp <- function(param, crit, simplices, volume, n) {

  ix <- rselect(n, volume)
  nobj <- dim(crit)[2]
  nvar <- dim(param)[2]

  xnew <- matrix(0, nrow = n, ncol = nvar)
  pcrit <- matrix(0, nrow = n, ncol = nobj)

  for (i in 1:n) {
    isimp <- ix[i]

    param_s <- param[simplices[isimp, ], ]
    crit_s <- crit[simplices[isimp, ], ]

    # Random drawing of a set of barycentric coordinates
    xb <- runif(nobj + 1)
    xb <- xb / sum(xb)

    xnew[i, ] <- xb %*% param_s # Coordinates in the space of the parameters
    pcrit[i, ] <- xb %*% crit_s  # Coordinates in the space of the criteria
  }
  return(list("xnew" = xnew, "pcrit" = pcrit))
}

#' Plotting of a population of objectives
#'
#' Plot graphs the population regarding each couple of objectives
#'
#' @param MatObj : matrix of the objectives [NInd, nobj]
#' @param nobj : number of objectives
#' @param ngen : number of generations (optional)
#' @param nrun : number of model evaluations (optional)
#' @param objnames : vector of objectives names (optional)
#' @param MatEvol : matrix of the evolution of the optimal objectives (optional)
#' @param popsize : integer, size of the initial population (optional)
#'
#' @examples
#' # Definition of the population
#' Pop <- matrix(runif(300), 100, 3)
#' # Call the function
#' plot_population(MatObj = Pop, nobj = 3, objnames = c("Obj1", "Obj2", "Obj3"))
#'
#' @author Celine Monteil

plot_population <- function(MatObj, nobj, ngen = NULL, nrun = NULL, objnames = NULL, MatEvol = NULL, popsize = 0) {

  # Graphs
  info <- paste("ngen=", ngen, ", nrun=", nrun, sep = "")
  nb_fen <- choose(n = nobj, k = 2) + 1
  if (nb_fen <= 4) {
    par(mfrow = c((nobj - 1), 2))
  } else {
    par(mfrow = c(3, floor(nb_fen / 3) + 1))
  }
  l <- seq(1, nobj)
  for (i_fig in 1:(nobj - 1)) {
    l_tmp <- l[-i_fig]
    l_tmp <- l_tmp[l_tmp > i_fig]
    for (i_fig2 in l_tmp) {
      plot(MatObj[, i_fig], MatObj[, i_fig2], xlab = objnames[i_fig], ylab = objnames[i_fig2])
    }
  }

  if (!is.null(MatEvol)) {
    plot(x = MatEvol[1, ], y = MatEvol[2, ],
         ylim = c(min(MatEvol[-1, ]), max(MatEvol[-1, ])),
         xlab = info, ylab = "Optimal Criteria", pch = 0)

    lapply(c(2:nobj), function(i) {
       points(x = MatEvol[1, ], y = MatEvol[i + 1, ], col = i, pch = (i - 1))
    })

    xlgd <- popsize + (nrun - popsize) * 2 / 3
    ylgd <- min(MatEvol[-1, ]) + (max(MatEvol[-1, ]) - min(MatEvol[-1, ])) / 2
    legend(xlgd, ylgd, legend = objnames, col = 1:nobj, pch = 0:(nobj - 1))
  }
}

#' Successive Pareto fronts of a population
#'
#' calculates the successive Pareto fronts of a population (classification "onion peel"), when objectives need to be maximized.
#'
#' @param matobj : matrix [ NInd , NObj ] of objectives
#' @return f : vector of dimension NInd of dominances
#'
#' @examples
#' # Definition of the parameters
#' matobj <- matrix(runif(200), 100, 2)
#' # Call the function
#' pareto_rank <- dominate(matobj)
#'
#' @author Alban de Lavenne, Fabrice Zaoui

dominate <- function(matobj) {

  # Test for possible NA values
  try(if(any(is.na(matobj))){
    message("The dominance cannot be calcultated due to a NA value")
    invokeRestart("abort")
  })

  if (!is.double(matobj)) {storage.mode(matobj) <- 'double'}
  nind <- dim(matobj)[1] # Number of individuals
  ord <- order(matobj[,1], matobj[,2], decreasing=TRUE) # order to make it faster
  res <- matrix(data = .Call(c_dominate, matobj[ord,]), nrow = nind)
  res[ord] <- res # original order
  return(res)

}

#' Generation of a new population of parameter sets following the five rules of caRamel
#'
#' generates a new population of parameter sets following the five rules of caRamel
#'
#' @param param : matrix [ Nvec , NPar ] of parameters of the current population
#' @param crit : matrix [ Nvec , NObj ] of associated criteria
#' @param isperf : vector of Booleans of length NObj, TRUE if maximization of the objective, FALSE otherwise
#' @param sp : variance a priori of the parameters
#' @param bounds : lower and upper bounds of parameters [ NPar , 2 ]
#' @param repart_gene : matrix of length 4 giving the number of games to be generated with each rule: 1 Interpolation in the simplexes of the front, 2 Extrapolation according to the directions of the edges "orthogonal" to the front, 3 Random draws with prescribed variance-covariance matrix, 4 Recombination by functional blocks
#' @param blocks : list of integer vectors containing function blocks of parameters
#' @param fireworks : boolean, TRUE if one tests a random variation on each parameter and each maximum of O.F.
#' @return xnew : matrix of new vectors [ sum(Repart_Gene) + eventually (nobj+1)*nvar if fireworks , NPar ]
#' @return project_crit: assumed position of the new vectors in the criteria space: [ sum(Repart_Gene)+ eventually (nobj+1)*nvar if fireworks , NObj ];
#'
#' @examples
#' # Definition of the parameters
#' param <- matrix(rexp(100), 100, 1)
#' crit <- matrix(rexp(200), 100, 2)
#' isperf <- c(FALSE, FALSE)
#' bounds <- matrix(data = 1, nrow = 1, ncol = 2)
#' bounds[, 1] <- -5 * bounds[, 1]
#' bounds[, 2] <- 10 * bounds[, 2]
#' sp <- (bounds[, 2] - bounds[, 1]) / (2 * sqrt(3))
#' repart_gene <- c(5, 5, 5, 5)
#' fireworks <- TRUE
#' blocks <- NULL
#' # Call the function
#' res <- newXval(param, crit, isperf, sp, bounds, repart_gene, blocks, fireworks)
#'
#' @author Fabrice Zaoui

newXval <-
  function(param,
           crit,
           isperf,
           sp,
           bounds,
           repart_gene,
           blocks,
           fireworks) {

    nobj <- dim(crit)[2]
    npar <- dim(param)[2]
    nvec <- dim(param)[1]
    a <- 3 / 8

    # Standardization of the different objectives
    obj <- matrix(0, nrow = dim(crit)[1], ncol = nobj)
    for (i in 1:nobj) {
      obj[, i] <- (val2rank(crit[, i], 3) - a) / (nvec + 1 - 2 * a)
    }
    obj[, !isperf] <- 1 - obj[, !isperf]

    Fo <- dominate(obj)
    param_arch <- as.matrix(param[Fo == 1, ])
    if (dim(param_arch)[2]<npar){param_arch <- t(param_arch)} # to keep nb_param columns
    obj_arch <- obj[Fo == 1, ]
    crit_arch <- crit[Fo == 1, ]

    n_inter <- repart_gene[1]  # Maximum number of new sets created by interpolation in simplexes
    n_extra <- repart_gene[2]  # Maximum number of new sets created by extrapolation on the directions of the edges
    n_cov <- repart_gene[3]    # Maximum number of new sets created by random draws with prescribed covariance matrix
    n_recomb <- repart_gene[4] # Maximum number of new games created by recombinations

    xnew <- NULL
    project_crit <- NULL

    #**************************************************************
    #       TRIANGULATION OF SPACE OF OBJECTIVES (IF NEEDED)      *
    #**************************************************************

    if (n_inter > 0 | n_extra > 0) {
      obj <- unique(obj)
      simplices <- suppressMessages(delaunayn(obj))

      # For each simplex calculation of the number of vertices belonging to the Pareto front
      nf <-
        apply((matrix(
          Fo[simplices],
          nrow = dim(simplices)[1],
          ncol = dim(simplices)[2]
        ) == 1), 1, sum)

      # Keep only the simplexes with at least one vertex on the Pareto front
      ix <- which(nf > 0)
      simplices <- simplices[ix, ]
      simplices <- matrix(data = simplices,ncol = (nobj + 1)) # To keep a matrix even if a single simplex
      nbsimp <- length(ix)

      # Decomposition in edges, calculation of the volume of the simplexes preserved
      na <- nobj * (nobj + 1) / 2 #number of edges in a NObj size simplex

      volume <- matrix(0, nrow = nbsimp, ncol = 1)
      oriedge <- NULL
      ledge <- NULL

      for (s in 1:nbsimp) {
        P <- simplices[s, ]
        S <- obj[P, ]
        volume[s] <- vol_splx(S)
        direc <- Dimprove(S, Fo[P])
        if (!is.null(direc$oriedge)) {
          oriedge <-
            rbind(oriedge,  matrix(P[direc$oriedge], nrow = dim(direc$oriedge)[1]))
          ledge <- c(ledge, direc$ledge)
        }
      }

      unik <- !duplicated(oriedge)
      oriedge <- oriedge[unik, ]
      ledge <- ledge[unik]
    }

    #**************************************************************
    #                 GENERATION OF NEW POINTS                    *
    #**************************************************************

    # New sets created by interpolation in simplexes
    if (n_inter > 0) {
      carain <- Cinterp(param, crit, simplices, volume, n_inter)
      xnew <- rbind(xnew, carain$xnew)
      project_crit <- rbind(project_crit, carain$pcrit)
    }

    # New sets created by extrapolation on the directions of the edges
    if (n_extra > 0) {
      caraex <- Cextrap(param, crit, oriedge, ledge, n_extra)
      xnew <- rbind(xnew, caraex$xnew)
      project_crit <- rbind(project_crit, caraex$pcrit)
    }

    # New sets created by random draws with prescribed covariance matrix
    if (n_cov > 0) {
      # Calculation of the variance-covariance matrix on a reference subpopulation
      iref <- sort(unique(c(simplices))) # Reference population: all vertex points of a "frontal" simplex
      xref <- as.matrix(param[iref, ])
      xcov <- Cusecovar(xref, sqrt(2), n_cov)
      critcov <- matrix(NaN, nrow = n_cov, ncol = nobj)
      xnew <- rbind(xnew, xcov)
      project_crit <- rbind(project_crit, critcov)
    }

    # New sets created with block recombinations
    if (dim(param_arch)[1] > 1 & n_recomb > 0) {
      xrecomb <- Crecombination(param_arch, blocks, n_recomb)
      critrec <- matrix(NaN, nrow = n_recomb, ncol = nobj)
      xnew <- rbind(xnew, xrecomb)
      project_crit <- rbind(project_crit, critrec)
    }

    # New sets created by independent variations of parameters
    if (fireworks) {
      if (sum(Fo == 1) == 1) {  # In the case or only 1 point on the front
        obj_arch <- matrix(obj_arch, 1, nobj)
        crit_arch <- matrix(crit_arch, 1, nobj)
      }

      sp <- matrix(sp, nrow = 1, ncol = length(sp))

      # Points maximizing each O.F. individually
      m <- apply(obj_arch, 2, max)
      ipp <- apply(obj_arch, 2, which.max)

      # Maxi-min ("central" point of the front)
      m <- max(apply(obj_arch, 1, min))
      maximin <- which.max(apply(obj_arch, 1, min))
      ipp <- c(ipp, maximin)

      for (i in seq_len(length(ipp))) {
        xcloud <-
          matrix(param_arch[ipp[i], ],
                 nrow = npar,
                 ncol = npar,
                 byrow = TRUE)
        ccloud <-
          matrix(crit_arch[ipp[i], ],
                 nrow = npar,
                 ncol = nobj,
                 byrow = TRUE)
        devi <- c(rnorm(npar) * sp)
        xcloud <-
          xcloud + diag(x = devi,
                        nrow = length(devi),
                        ncol = length(devi))
        xcloud <- xcloud[sp > 0, ]
        ccloud <- ccloud[sp > 0, ]
        xnew <- rbind(xnew, xcloud)
        project_crit <- rbind(project_crit, ccloud)
      }

    }

    minp <-
      matrix(bounds[, 1],
             nrow = dim(xnew)[1],
             ncol = npar,
             byrow = TRUE)
    maxp <-
      matrix(bounds[, 2],
             nrow = dim(xnew)[1],
             ncol = npar,
             byrow = TRUE)
    ll <- which(xnew > maxp, arr.ind = TRUE)
    nelem <- dim(ll)[1]
    if (nelem > 0) {
      for (i in 1:nelem) {
        rnum <- ll[i, 1]
        cnum <- ll[i, 2]
        xnew[rnum, cnum] <- maxp[rnum, cnum]
      }
    }
    ll <- which(xnew < minp, arr.ind = TRUE)
    nelem <- dim(ll)[1]
    if (nelem > 0) {
      for (i in 1:nelem) {
        rnum <- ll[i, 1]
        cnum <- ll[i, 2]
        xnew[rnum, cnum] <- minp[rnum, cnum]
      }
    }

    return(list("xnew" = xnew, "pcrit" = project_crit))
  }

---
title: "Compute several Pareto fronts for a better global result"
author: "Fabrice Zaoui"
date: "March 10 2021"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Merging multi caRamel results}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

```{r caRa}
library(caRamel)
```

The [*Kursawe*](https://en.wikipedia.org/wiki/File:Kursawe_function.pdf) test function to optimize has two objectives and three variables.

```{r kursawe}
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
  k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
  return(c(k1, k2))
}
```

For instance, the following **caRamel** parameters for all the Kursawe optimizations can be:

```{r parameters}
nvar <- 3  # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]
nobj <- 2  # number of objectives
minmax <- c(FALSE, FALSE) # minimization for both objectives
popsize <- 100  # size of the genetic population
archsize <- 100  # size of the archive for the Pareto front
maxrun <- 1000  # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj)  # convergence criteria
```

# Multi-caRamel optimization

In this part we will run caRamel three times on the Kursawe test function and save all the front results.

```{r multi}
nrepeat <- 3  # number of calls to caRamel
concat_results_objectives <- NULL  # save results for all the calls
concat_results_parameters <- NULL

for (i in seq(nrepeat)) {
   optres <- caRamel(nobj,
               nvar,
               minmax,
               bounds,
               kursawe,
               popsize,
               archsize,
               maxrun,
               prec,
               carallel = 0,
               graph = FALSE,
               verbose = FALSE)
   
   concat_results_objectives <- rbind(concat_results_objectives,
                                      optres$objectives)
   concat_results_parameters <- rbind(concat_results_parameters,
                                      optres$parameters)
}
```

Then all the results are reduced using the *pareto* function in order to get a new global front:

```{r merge}
results_objectives <- concat_results_objectives
results_objectives[, !minmax] <- -results_objectives[, !minmax]  # important !

is_pareto <- pareto(results_objectives)  # mask

global_results_objectives <- concat_results_objectives[is_pareto, ]  # front from the three previous fronts
global_results_parameters <- concat_results_parameters[is_pareto, ]
```

# Results

All the results can now be plotted:

```{r plot}
plot(concat_results_objectives[, 1], concat_results_objectives[, 2],
     main = "Kursawe Pareto fronts",
     xlab = "Objective #1", ylab = "Objective #2")
points(global_results_objectives[, 1], global_results_objectives[, 2],
       col = "red", pch = "*")
```

---
title: "Sensitivity of the Pareto front"
author: "Fabrice Zaoui"
date: "September 16 2020"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Pareto front sensitivity}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

It is possible to compute the first order derivatives of the Pareto front with **caRamel** by setting the logical parameter _sensitivity_ to TRUE.

```{r caRa}
library(caRamel)
```

# Test function

## Schaffer

[*Schaffer*](https://en.wikipedia.org/wiki/File:Schaffer_function_1.pdf) test function has two objectives with one variable.

```{r schaffer}
schaffer <- function(i) {
  s1 <- x[i,1] * x[i,1]
  s2 <- (x[i,1] - 2) * (x[i,1] - 2)
  return(c(s1, s2))
}
```

Note that :

* parameter _i_ is mandatory for the management of parallelism.
* the variable __must be named__ _x_ and is a matrix of size [npopulation, nvariables].

For instance, the variable will lie in the range [-10, 10]:

```{r schaffer_variable}
nvar <- 1 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -10 * bounds[, 1]
bounds[, 2] <- 10 * bounds[, 2]
```

Both functions are to be minimized:

```{r schaffer_objectives}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
```

Before calling **caRamel** in order to optimize the Schaffer's problem, some algorithmic parameters need to be set:

```{r schaffer_param}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
```

## Optimization

Then the minimization problem can be launched with a sensitivity analysis:

```{r schaffer_launch, fig.show="hide", results="hide"}
results <-
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          schaffer,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE, 
          sensitivity=TRUE) # sensitivity required
```

Test if the convergence is successful:

```{r schaffer_OK}
print(results$success==TRUE)
```

Plot the Pareto front:

```{r schaffer_plot1}
plot(results$objectives[,1], results$objectives[,2], main="Schaffer Pareto front", xlab="Objective #1", ylab="Objective #2")
```

```{r schaffer_plot2}
plot(results$parameters, main="Corresponding values for X", xlab="Element of the archive", ylab="X Variable")
```

## Sensitivity

The sensitivity of the Pareto front is evalutated by computing first order derivatives.
For each of the objective, one Jacobian matrix is computed:

```{r jacobian}
names(results$derivatives)
```

Plot the sensitivity for the first objective:
```{r sensi}
plot(results$parameters, results$derivatives$Jacobian_1, main="Sensitivitiy for the first objective", ylab="Sensitivity values", xlab="X values")
```

Plot the histogram for the second objective:

```{r histo}
hist(results$derivatives$Jacobian_2, main="Sensitivitiy for the second objective", xlab="Sensitivity values", ylab="Distribution of the Pareto front")
```

Plot the sensitivity of the Pareto front for the two objectives:
```{r sensi2}
plot(results$derivatives$Jacobian_1, results$derivatives$Jacobian_2, main="Sensitivitiy for both objectives", ylab="Sensitivity values #2", xlab="Sensitivity values #1")
```

---
title: "Using a Python function with caRamel"
author: "Fabrice Zaoui"
date: "September 29 2020"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{caRamel and Python function}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)
```
## Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

```{r caRa, eval=F, echo=T}
library(caRamel)
```

This example will use the **reticulate** package in order to call a Python function from R. Download the package from CRAN and then install and load it

```{r reti, eval=F, echo=T}
library(reticulate)
```

[*Kursawe*](https://en.wikipedia.org/wiki/File:Kursawe_function.pdf) test function has two objectives of three variables. This function will be written in a Python script named *kursawe.py* with the following content:

```{python kursawe, eval=F, echo=T}
import numpy as np

def kursawe(x):
    k1 = -10 * np.exp(-0.2 * np.sqrt(x[0]**2 + x[1]**2)) - \
        10 * np.exp(-0.2 * np.sqrt(x[1]**2 + x[2]**2))
    k2 = np.abs(x[0])**0.8 + 5 * np.sin(x[0]**3) + np.abs(x[1])**0.8 +\
        5 * np.sin(x[1]**3) + np.abs(x[2])**0.8 + 5 * np.sin(x[2]**3)
    return np.array([k1, k2])
```

The Python function has to be loaded in R:

```{r load, eval=F, echo=T}
use_python("/usr/local/bin/python3")
source_python("kursawe.py")
```


This function is not directly called from **caRamel** but with a new wrapper function and finally all can be gathered in it (recommended):

```{r wrap, eval=F, echo=T}
wrapperFunction <- function(i) {
  # load the package
  library(reticulate)
  # python path
  use_python("/usr/local/bin/python3")
  # source the Python function
  source_python("kursawe.py")
  # call the Python function and return the results
  return(kursawe(x[i,]))
}
```

The variables lie in the range [-5, 5]:

```{r kursawe_variable, eval=F, echo=T}
nvar <- 3 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]
```

Both functions are to be minimized:

```{r kursawe_objectives, eval=F, echo=T}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
```

Set algorithmic parameters and launch **caRamel**:

```{r kursawe_param, , eval=F, echo=T, fig.show="hide", results="hide"}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

results <- 
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          wrapperFunction, # It's the wrapper function that will be called
          popsize,
          archsize,
          maxrun,
          prec)
```

Test if the convergence is successful and plot the optimal front:

```{r kursawe_OK_plot, eval=F, echo=T}
print(results$success==TRUE)

plot(results$objectives[,1], results$objectives[,2], main="Kursawe Pareto front", xlab="Objective #1", ylab="Objective #2")
```

Finally plot the convergences of the objective functions:

```{r kursawe_plot_conv, eval=F, echo=T}
matplot(results$save_crit[,1],cbind(results$save_crit[,2],results$save_crit[,3]),type="l",col=c("blue","red"), main="Convergence", xlab="Number of calls", ylab="Objectives values")
```

---
title: "Three ways to call the user functions"
author: "Fabrice Zaoui"
date: "October 01 2020"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Options for the carallel parameter}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

```{r caRa}
library(caRamel)
```

There are three possible choices on how **caRamel** can call the R user functions. Each choice is parameterized by the *carallel* value.

# Sequential or parallel mode

The computation of the user functions can be done sequentially or in parallel for each individual of the genetic population according to the choice of *carallel* when calling **caRamel**.

The evaluation of the population is done in a sequential mode if *carallel* is 0, or in parallel if *carallel* is 1 (this last one is the default option).

For these two options, the R user function takes a single input parameter *i* giving the number of the individual of the population *x*. For instance:

```{r kursawe1}
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
  k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
  return(c(k1, k2))
}
```

Two objectives are evaluated here and a vector of the corresponding values is returned for the individual *x[i,]*.

# User-defined mode

If the value of *carallel* is 2 then the entire population is given to the R user function and one has to decide how to evaluate it. For instance hereafter with a simple for-loop:

```{r kursawe2}
kursawe <- function(x) { # receipt of the entire population
  popsize <- dim(x)[1] # size of the population to evaluate
  nobj <- 2 # number of objectives
  results <- matrix(0, nrow = popsize, ncol = nobj) # matrix of results
  for(i in 1:popsize){
      k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
      k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
      results[i,] <- c(k1, k2)
  }
  return(results)
}
```

---
title: "Multi-caRamel optimization with MPI"
author: "Fabrice Zaoui"
date: "March 02 2021"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Pareto front of fronts with MPI}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(python.reticulate = FALSE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

```{r caRa, eval=F, echo=T}
library(caRamel)
```

This example will use the **pbdMPI** package in order to use several processes of caRamel. Download the package from CRAN and then install and load it. Make also sure that you have an MPI distribution installed on your system, see for instance [*Open MPI*](https://www.open-mpi.org) if necessary.

```{r pbdMPI, eval=F, echo=T}
library(pbdMPI)
```

[*Kursawe*](https://en.wikipedia.org/wiki/File:Kursawe_function.pdf) test function has two objectives of three variables.

```{r kursawe, eval=F, echo=T}
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1]^2 + x[i,2]^2)) - 10 * exp(-0.2 * sqrt(x[i,2]^2 + x[i,3]^2))
  k2 <- abs(x[i,1])^0.8 + 5 * sin(x[i,1]^3) + abs(x[i,2])^0.8 + 5 * sin(x[i,2]^3) + abs(x[i,3])^0.8 + 5 * sin(x[i,3]^3)
  return(c(k1, k2))
}
```

The variables lie in the range [-5, 5]:

```{r kursawe_variable, eval=F, echo=T}
nvar <- 3 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]
```

Both functions are to be minimized:

```{r kursawe_objectives, eval=F, echo=T}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # minimization for both functions
```

Set algorithmic parameters for **caRamel**:

```{r kursawe_param, eval=F, echo=T}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
```

# Combining distributed and shared memory parallelism

The **caRamel** parameters *carallel* and *numcores* help to define the shared memory parallelism used to evaluate by the user's function each member of the genetic population.

The shared memory parallelism runs on a single computational node (or workstation). For instance 20 cores can be defined with the following call:

```{r call_caRa, eval=F, echo=T}
results <- 
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          kursawe,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel = 1,
          numcores = 20,
          graph = FALSE,
          verbose = FALSE)
```

For calling several times **caRamel** on a distributed memory cluster of computers (or nodes) MPI is used.

At the end of the script, the different results of optimizations can be gathered with:

```{r mpi_caRa, eval=F, echo=T}
init()  # MPI functions from the pbdMPI package
size <- comm.size()
rank <- comm.rank()

results <- gather(optres, rank.dest = 0)  # gather all results on the main process

if (rank == 0) saveRDS(results, "Results.Rds")  # save all the results on disk

finalize()
```

Finally, the previous script can be launched in parallel on a number of nodes, for instance 4 nodes:

```{bash launch, eval=F, echo=T}
mpirun -n 4 Rscript --vanilla my_kursawe_optim.R 
```

---
title: "Dealing with constraints"
author: "Fabrice Zaoui"
date: "October 05 2018"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{A constrained problem}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

Dealing with constraints is possible with **caRamel** by returning a NaN value for an infeasible solution. See the example below.

```{r caRa}
library(caRamel)
```

# Test functions

## Constr-Ex problem

[*Constr-Ex*](https://en.wikipedia.org/wiki/File:Constr-Ex_problem.pdf) test function has two objectives with two variables and two inequality constraints.

```{r constr}
constr_ex <- function(i) {
  # functions f1 and f2
  s1 <- x[i,1]
  s2 <- (1. + x[i,2]) / x[i,1]
  # now test for the feasibility
  # constraint g1
  if((x[i,2] + 9. * x[i,1] - 6.) < 0. | (-x[i,2] + 9. * x[i,1] -1.) < 0.) {
    s1 <- NaN
    s2 <- NaN
  }
  return(c(s1, s2))
}
```

Note that :

* parameter _i_ is mandatory for the management of parallelism.
* the variable __must be named__ _x_ and is a matrix of size [npopulation, nvariables].

The variable lies in the range [0.1, 1] and [0, 5]:

```{r constr_variable}
nvar <- 2 # number of variables
bounds <- matrix(data = 0., nrow = nvar, ncol = 2) # upper and lower bounds
bounds[1, 1] <- 0.1
bounds[1, 2] <- 1.
bounds[2, 1] <- 0.
bounds[2, 2] <- 5.
```

Both functions are to be minimized:

```{r constr_objectives}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
```

Before calling **caRamel** in order to optimize the Constr_Ex problem, some algorithmic parameters need to be set:

```{r constr_param}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
```

Then the minimization problem can be launched:

```{r schaffer_launch, fig.show="hide", results="hide"}
results <-
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          constr_ex,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE) # no parallelism
```

Test if the convergence is successful:

```{r schaffer_OK}
print(results$success==TRUE)
```

Plot the Pareto front:

```{r schaffer_plot1}
plot(results$objectives[,1], results$objectives[,2], main="Constr_Ex Pareto front", xlab="Objective #1", ylab="Objective #2")
```

```{r schaffer_plot2}
plot(results$parameters, main="Corresponding values for X", xlab="Element of the archive", ylab="X Variable")
```

---
title: "Using caRamel on two benchmark tests"
author: "Fabrice Zaoui"
date: "January 19 2018"
output: html_document
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Simple test functions}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Short Description

**caRamel** is a multiobjective evolutionary algorithm combining the MEAS algorithm and the NGSA-II algorithm.

Download the package from CRAN or [GitHub](https://github.com/fzao/caRamel) and then install and load it.

```{r caRa}
library(caRamel)
```

# Test functions

## Schaffer

[*Schaffer*](https://en.wikipedia.org/wiki/File:Schaffer_function_2_-_multi-objective.pdf) test function has two objectives with one variable.

```{r schaffer}
schaffer <- function(i) {
  if (x[i,1] <= 1) {
    s1 <- -x[i,1]
  } else if (x[i,1] <= 3) {
    s1 <- x[i,1] - 2
  } else if (x[i,1] <= 4) {
    s1 <- 4 - x[i,1]
  } else {
    s1 <- x[i,1] - 4
  }
  s2 <- (x[i,1] - 5) * (x[i,1] - 5)
  return(c(s1, s2))
}
```

Note that :

* parameter _i_ is mandatory for the management of parallelism.
* the variable __must be named__ _x_ and is a matrix of size [npopulation, nvariables].

The variable lies in the range [-5, 10]:

```{r schaffer_variable}
nvar <- 1 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 10 * bounds[, 2]
```

Both functions are to be minimized:

```{r schaffer_objectives}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
```

Before calling **caRamel** in order to optimize the Schaffer's problem, some algorithmic parameters need to be set:

```{r schaffer_param}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase
```

Then the minimization problem can be launched:

```{r schaffer_launch, fig.show="hide", results="hide"}
results <-
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          schaffer,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE) # no parallelism
```

Test if the convergence is successful:

```{r schaffer_OK}
print(results$success==TRUE)
```

Plot the Pareto front:

```{r schaffer_plot1}
plot(results$objectives[,1], results$objectives[,2], main="Schaffer Pareto front", xlab="Objective #1", ylab="Objective #2")
```

```{r schaffer_plot2}
plot(results$parameters, main="Corresponding values for X", xlab="Element of the archive", ylab="X Variable")
```

## Kursawe

[*Kursawe*](https://en.wikipedia.org/wiki/File:Kursawe_function.pdf) test function has two objectives of three variables.

```{r kursawe}
kursawe <- function(i) {
  k1 <- -10 * exp(-0.2 * sqrt(x[i,1] ^ 2 + x[i,2] ^ 2)) - 10 * exp(-0.2 * sqrt(x[i,2] ^2 + x[i,3] ^ 2))
  k2 <- abs(x[i,1]) ^ 0.8 + 5 * sin(x[i,1] ^ 3) + abs(x[i,2]) ^ 0.8 + 5 * sin(x[i,2] ^3) + abs(x[i,3]) ^ 0.8 + 5 * sin(x[i,3] ^ 3)
  return(c(k1, k2))
}
```

The variables lie in the range [-5, 5]:

```{r kursawe_variable}
nvar <- 3 # number of variables
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # upper and lower bounds
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 5 * bounds[, 2]
```

Both functions are to be minimized:

```{r kursawe_objectives}
nobj <- 2 # number of objectives
minmax <- c(FALSE, FALSE) # min and min
```

Set algorithmic parameters and launch **caRamel**:

```{r kursawe_param, fig.show="hide", results="hide"}
popsize <- 100 # size of the genetic population
archsize <- 100 # size of the archive for the Pareto front
maxrun <- 1000 # maximum number of calls
prec <- matrix(1.e-3, nrow = 1, ncol = nobj) # accuracy for the convergence phase

results <- 
  caRamel(nobj,
          nvar,
          minmax,
          bounds,
          kursawe,
          popsize,
          archsize,
          maxrun,
          prec,
          carallel=FALSE) # no parallelism
```

Test if the convergence is successful and plot the optimal front:

```{r kursawe_OK_plot}
print(results$success==TRUE)

plot(results$objectives[,1], results$objectives[,2], main="Kursawe Pareto front", xlab="Objective #1", ylab="Objective #2")
```

Finally plot the convergences of the objective functions:

```{r kursawe_plot_conv}
matplot(results$save_crit[,1],cbind(results$save_crit[,2],results$save_crit[,3]),type="l",col=c("blue","red"), main="Convergence", xlab="Number of calls", ylab="Objectives values")
```

# References

* Efstratiadis, A. and Koutsoyiannis, D., _The multiobjective evolutionary annealing-simplex method and its application in calibrating hydrological models_, EGU General Assembly 2005, Geophysical Research Abstracts, vol.7, Vienna, European Geophysical Union
* Reed, P. and Devireddy, D., _Groundwater monitoring design: a case study combining epsilon-dominance archiving and automatic parameterization for the NGSA-II_, Coello-Coello C editor, Applications of multiobjective evolutionary algorithms, Advances in natural computation series, vol. 1, pp. 79-100, Word Scientific, New-York, 2004

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caRamel.R
\name{caRamel}
\alias{caRamel}
\title{MAIN FUNCTION: multi-objective optimizer}
\usage{
caRamel(
  nobj,
  nvar,
  minmax,
  bounds,
  func,
  popsize,
  archsize,
  maxrun,
  prec,
  repart_gene = c(5, 5, 5, 5),
  gpp = NULL,
  blocks = NULL,
  pop = NULL,
  funcinit = NULL,
  objnames = NULL,
  listsave = NULL,
  write_gen = FALSE,
  carallel = 1,
  numcores = NULL,
  graph = TRUE,
  sensitivity = FALSE,
  verbose = TRUE,
  worklist = NULL
)
}
\arguments{
\item{nobj}{: (integer, length = 1) the number of objectives to optimize (nobj >= 2)}

\item{nvar}{: (integer, length = 1) the number of variables}

\item{minmax}{: (logical, length = nobj) the objective is either a minimization (FALSE value) or a maximization (TRUE value)}

\item{bounds}{: (matrix, nrow = nvar, ncol = 2) lower and upper bounds for the variables}

\item{func}{: (function) the objective function to optimize. Input argument is the number of parameter set (integer) in the x matrix. The function has to return a vector of at least 'nobj' values (Objectives 1 to nobj are used for optimization, values after nobj are recorded for information.).}

\item{popsize}{: (integer, length = 1) the population size for the genetic algorithm}

\item{archsize}{: (integer, length = 1) the size of the Pareto front}

\item{maxrun}{: (integer, length = 1) the max. number of simulations allowed}

\item{prec}{: (double, length = nobj) the desired accuracy for the optimization of the objectives}

\item{repart_gene}{: (integer, length = 4) optional, number of new parameter sets for each rule and per generation}

\item{gpp}{: (integer, length = 1) optional, calling frequency for the rule "Fireworks"}

\item{blocks}{(optional): groups for parameters}

\item{pop}{: (matrix, nrow = nset, ncol = nvar or nvar+nobj ) optional, initial population (used to restart an optimization)}

\item{funcinit}{(function, optional): the initialization function applied on each node of cluster when parallel computation. The arguments are cl and numcores}

\item{objnames}{(optional): names of the objectives}

\item{listsave}{(optional): names of the listing files. Default: None (no output). If exists, fields to be defined: "pmt" (file of parameters on the Pareto Front), "obj" (file of corresponding objective values), "evol" (evolution of maximum objectives by generation). Optional field: "totalpop" (total population and corresponding objectives, useful to restart a computation)}

\item{write_gen}{: (logical, length = 1) optional, if TRUE, save files 'pmt' and 'obj' at each generation (FALSE by default)}

\item{carallel}{: (integer, length = 1) optional, do parallel computations? (0: sequential, 1:parallel (default) , 2:user-defined choice)}

\item{numcores}{: (integer, length = 1) optional, the number of cores for the parallel computations (all cores by default)}

\item{graph}{: (logical, length = 1) optional, plot graphical output at each generation (TRUE by default)}

\item{sensitivity}{: (logical, length = 1) optional, compute the first order derivatives of the pareto front (FALSE by default)}

\item{verbose}{: (logical, length = 1) optional, verbosity mode (TRUE by default)}

\item{worklist}{: optional values to be transmitted to the user's function (not used)}
}
\value{
List of seven elements:
\describe{
\item{success}{return value (logical, length = 1) : TRUE if successfull}
\item{parameters}{Pareto front (matrix, nrow = archsize, ncol = nvar)}
\item{objectives}{objectives of the Pareto front (matrix, nrow = archsize, ncol = nobj+nadditional)}
\item{derivatives}{list of the Jacobian matrices of the Pareto front if the sensitivity parameter is TRUE or NA otherwise}
\item{save_crit}{evolution of the optimal objectives}
\item{total_pop}{total population (matrix, nrow = popsize+archsize, ncol = nvar+nobj+nadditional)}
\item{gpp}{the calling period for the third generation rule (independent sampling with a priori parameters variance)}
}
}
\description{
Multi-objective optimizer. It requires to define a multi-objective function (func) to calibrate the model and bounds on the parameters to optimize.
}
\details{
The optimizer was originally written for Scilab by Nicolas Le Moine.
The algorithm is a hybrid of the MEAS algorithm (Efstratiadis and Koutsoyiannis (2005) <doi:10.13140/RG.2.2.32963.81446>) by using the directional search method based on the simplexes of the objective space
    and the epsilon-NGSA-II algorithm with the method of classification of the parameter vectors archiving management by epsilon-dominance (Reed and Devireddy <doi:10.1142/9789812567796_0004>).
Reference : "Multi-objective calibration by combination of stochastic and gradient-like parameter generation rules – the caRamel algorithm"
            Celine Monteil (EDF), Fabrice Zaoui (EDF), Nicolas Le Moine (UPMC) and Frederic Hendrickx (EDF)
            June 2020 Hydrology and Earth System Sciences 24(6):3189-3209
            DOI: 10.5194/hess-24-3189-2020
Documentation : "Principe de l'optimiseur CaRaMEL et illustration au travers d'exemples de parametres dans le cadre de la modelisation hydrologique conceptuelle"
                Frederic Hendrickx (EDF) and Nicolas Le Moine (UPMC)
                Report EDF H-P73-2014-09038-FR
}
\examples{
# Definition of the test function
viennet <- function(i) {
  val1 <- 0.5*(x[i,1]*x[i,1]+x[i,2]*x[i,2])+sin(x[i,1]*x[i,1]+x[i,2]*x[i,2])
  val2 <- 15+(x[i,1]-x[i,2]+1)*(x[i,1]-x[i,2]+1)/27+(3*x[i,1]-2*x[i,2]+4)*(3*x[i,1]-2*x[i,2]+4)/8
  val3 <- 1/(x[i,1]*x[i,1]+x[i,2]*x[i,2]+1) -1.1*exp(-(x[i,1]*x[i,1]+x[i,2]*x[i,2]))
  return(c(val1,val2,val3))
}
# Number of objectives
nobj <- 3
# Number of variables
nvar <- 2
# All the objectives are to be minimized
minmax <- c(FALSE, FALSE, FALSE)
# Define the bound constraints
bounds <- matrix(data = 1, nrow = nvar, ncol = 2)
bounds[, 1] <- -3 * bounds[, 1]
bounds[, 2] <- 3 * bounds[, 2]

# Caramel optimization
results <-
  caRamel(nobj = nobj,
          nvar = nvar,
          minmax =  minmax,
          bounds = bounds,
          func = viennet,
          popsize = 100,
          archsize = 100,
          maxrun = 500,
          prec = matrix(1.e-3, nrow = 1, ncol = nobj),
          carallel = 0)

}
\author{
Fabrice Zaoui - Celine Monteil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downsize.R
\name{downsize}
\alias{downsize}
\title{Downsizing of a population to only one individual per box up to a given accuracy}
\usage{
downsize(points, Fo, prec)
}
\arguments{
\item{points}{: matrix of objectives}

\item{Fo}{: rank on the front of each point (1: dominates on the Pareto)}

\item{prec}{: (double, length = nobj) desired accuracy for sorting objectives}
}
\value{
vector indices
}
\description{
reduces the number of individuals in a population to only one individual per box up to a given accuracy
}
\examples{
# Definition of the parameters
points <- matrix(rexp(200), 100, 2)
prec <- c(1.e-3, 1.e-3)
Fo <- sample(1:100, 100)
# Call the function
res <- downsize(points, Fo, prec)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cusecovar.R
\name{Cusecovar}
\alias{Cusecovar}
\title{New parameter vectors generation respecting a covariance structure}
\usage{
Cusecovar(xref, amplif, n)
}
\arguments{
\item{xref}{: matrix [ . , NPar ] of the reference population whose covariance structure is to be used}

\item{amplif}{: amplification factor of the standard deviation on each parameter}

\item{n}{: number of new vectors to generate}
}
\value{
xnew : matrix [ n , NPar ] of new vectors
}
\description{
proposes new parameter vectors respecting a covariance structure
}
\examples{
# Definition of the parameters
xref <- matrix(rexp(35), 35, 1)
amplif <- 2.
n <- 5
# Call the function
res <- Cusecovar(xref, amplif, n)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Crecombination.R
\name{Crecombination}
\alias{Crecombination}
\title{Recombination of the sets of parameters}
\usage{
Crecombination(param, blocks, n)
}
\arguments{
\item{param}{: matrix [ . , NPar ] of the population of parameters}

\item{blocks}{: list of integer vectors: list of variable blocks for recombination}

\item{n}{: number of new vectors to generate}
}
\value{
xnew : matrix [ n , NPar ] of new vectors
}
\description{
performs a recombination of the sets of parameters
}
\examples{
# Definition of the parameters
param <- matrix(rexp(15), 15, 1)
blocks <- NULL
n <- 5
# Call the function
res <- Crecombination(param, blocks, n)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dominated.R
\name{dominated}
\alias{dominated}
\title{Rows domination of a matrix by a vector}
\usage{
dominated(x, Y)
}
\arguments{
\item{x}{: row vecteur}

\item{Y}{: matrix}
}
\value{
D : vector of booleans
}
\description{
indicates which rows of the matrix Y are dominated by the vector (row) x
}
\examples{
# Definition of the parameters
Y <- matrix(rexp(200), 100, 2)
x <- Y[1,]
# Call the function
res <- dominated(x, Y)

}
\author{
Alban de Lavenne, Fabrice Zaoui
}

\name{caRamel-package}
\alias{caRamel-package}
\docType{package}

\title{caRamel optimizer}

\description{Automatic Calibration by Evolutionary Multi Objective Algorithm}

\details{
\emph{caRamel} is a package for multi-objective optimization of complex environmental models.

The algorithm is a hybrid of the \emph{MEAS} algorithm (Efstratiadis and Koutsoyiannis, 2005)
by using the directional search method based on the simplexes of the objective space 
and the \emph{epsilon-NGSA-II} algorithm with the method of classification of the parameter vectors archiving management by epsilon-dominance
(Reed and Devireddy, 2004).

The main function of the package is \emph{caRamel()}.

This function uses all the other functions of the package.

An example of an hydrological optimization is available on the following presentation: \href{https://www.researchgate.net/publication/341931417_Automatic_Calibration_by_Evolutionary_Multi_Objective_Algorithm_the_caRamel_R_package}{useR! 2019}

}
\author{Fabrice Zaoui, Nicolas Le Moine, Celine Monteil (EDF R&D - LNHE)}
\references{
   
   Efstratiadis, A. and Koutsoyiannis, D. (2005) \emph{The multi-objective evolutionary annealing-simplex method and its application in calibration hydrological models},
   in EGU General Assembly 2005, Geophysical Research Abstracts, Vol. 7, Vienna, 04593, European Geophysical Union. doi:10.13140/ RG.2.2.32963.81446.
   
   Le Moine, N. (2009) \emph{Description d’un algorithme génétique multi-objectif pour la calibration d’un modèle pluie-débit} (in French).
   Post-Doctoral Status Rep. 2, UPMC/EDF, 13 pp.
   
   Reed, P. and Devireddy, D. (2004) \emph{Groundwater monitoring design: a case study combining epsilon-dominance archiving and automatic parameterization for the NSGA-II},
   in Coello-Coello C, editor. Applications of multi-objective evolutionary algorithms, Advances in natural computation series,
   vol. 1, pp. 79-100, World Scientific, New York. doi:10.1142/9789812567796_0004.
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dominate.R
\name{dominate}
\alias{dominate}
\title{Successive Pareto fronts of a population}
\usage{
dominate(matobj)
}
\arguments{
\item{matobj}{: matrix [ NInd , NObj ] of objectives}
}
\value{
f : vector of dimension NInd of dominances
}
\description{
calculates the successive Pareto fronts of a population (classification "onion peel"), when objectives need to be maximized.
}
\examples{
# Definition of the parameters
matobj <- matrix(runif(200), 100, 2)
# Call the function
pareto_rank <- dominate(matobj)

}
\author{
Alban de Lavenne, Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/newXval.R
\name{newXval}
\alias{newXval}
\title{Generation of a new population of parameter sets following the five rules of caRamel}
\usage{
newXval(param, crit, isperf, sp, bounds, repart_gene, blocks, fireworks)
}
\arguments{
\item{param}{: matrix [ Nvec , NPar ] of parameters of the current population}

\item{crit}{: matrix [ Nvec , NObj ] of associated criteria}

\item{isperf}{: vector of Booleans of length NObj, TRUE if maximization of the objective, FALSE otherwise}

\item{sp}{: variance a priori of the parameters}

\item{bounds}{: lower and upper bounds of parameters [ NPar , 2 ]}

\item{repart_gene}{: matrix of length 4 giving the number of games to be generated with each rule: 1 Interpolation in the simplexes of the front, 2 Extrapolation according to the directions of the edges "orthogonal" to the front, 3 Random draws with prescribed variance-covariance matrix, 4 Recombination by functional blocks}

\item{blocks}{: list of integer vectors containing function blocks of parameters}

\item{fireworks}{: boolean, TRUE if one tests a random variation on each parameter and each maximum of O.F.}
}
\value{
xnew : matrix of new vectors [ sum(Repart_Gene) + eventually (nobj+1)*nvar if fireworks , NPar ]

project_crit: assumed position of the new vectors in the criteria space: [ sum(Repart_Gene)+ eventually (nobj+1)*nvar if fireworks , NObj ];
}
\description{
generates a new population of parameter sets following the five rules of caRamel
}
\examples{
# Definition of the parameters
param <- matrix(rexp(100), 100, 1)
crit <- matrix(rexp(200), 100, 2)
isperf <- c(FALSE, FALSE)
bounds <- matrix(data = 1, nrow = 1, ncol = 2)
bounds[, 1] <- -5 * bounds[, 1]
bounds[, 2] <- 10 * bounds[, 2]
sp <- (bounds[, 2] - bounds[, 1]) / (2 * sqrt(3))
repart_gene <- c(5, 5, 5, 5)
fireworks <- TRUE
blocks <- NULL
# Call the function
res <- newXval(param, crit, isperf, sp, bounds, repart_gene, blocks, fireworks)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pareto.R
\name{pareto}
\alias{pareto}
\title{Indicates which rows are Pareto}
\usage{
pareto(X)
}
\arguments{
\item{X}{: matrix of objectives [NInd * NObj]}
}
\value{
Ft : vector [NInd], TRUE when the set is on the Pareto front.
}
\description{
indicates which rows of the X criterion matrix are Pareto, when objectives need to be maximized
}
\examples{
# Definition of the parameters
X <- matrix(runif(200), 100, 2)
# Call the function
is_pareto <- pareto(X)

}
\author{
Alban de Lavenne, Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cextrap.R
\name{Cextrap}
\alias{Cextrap}
\title{Extrapolation along orthogonal directions to the Pareto front in the space of the objectives}
\usage{
Cextrap(param, crit, directions, longu, n)
}
\arguments{
\item{param}{: matrix [ NPoints , NPar ] of already evaluated parameters}

\item{crit}{: matrix [ Npoints , NObj ] of associated criteria}

\item{directions}{: matrix [ NDir, 2 ] the starting and ending points of the candidate vectors}

\item{longu}{: matrix [ NDir , 1 ] giving the length of each segment thus defined in the OBJ space (measure of the probability of exploring this direction)}

\item{n}{: number of new vectors to generate}
}
\value{
xnew : matrix [ n , NPar ] of new vectors

pcrit : matrix [ n , NObj ] estimated positions of new sets in the goal space
}
\description{
gives n new candidates by extrapolation along orthogonal directions to the Pareto front in the space of the objectives
}
\examples{
# Definition of the parameters
param <- matrix(rexp(100), 100, 1)
crit <- matrix(rexp(200), 100, 2)
directions <- matrix(c(1,3,2,7,13,40), nrow = 3, ncol = 2)
longu <- runif(3)
n <- 5
# Call the function
res <- Cextrap(param, crit, directions, longu, n)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/val2rank.R
\name{val2rank}
\alias{val2rank}
\title{Converting the values of a vector into their rank}
\usage{
val2rank(X, opt)
}
\arguments{
\item{X}{: vector to treat}

\item{opt}{: integer which gives the rule to follow in case of tied ranks (repeated values): if opt = 1, one returns the average rank, if opt = 2, one returns the corresponding rank in the series of the unique values, if opt = 3, return the max rank}
}
\value{
R : rank vector
}
\description{
converts the values of a vector into their rank
}
\examples{
# Definition of the parameters
X <- matrix(rexp(100), 100, 1)
opt <- 3
# Call the function
res <- val2rank(X, opt)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matvcov.R
\name{matvcov}
\alias{matvcov}
\title{Calculation of the variances-covariances matrix on the reference population}
\usage{
matvcov(x, g)
}
\arguments{
\item{x}{: population}

\item{g}{: center of reference population (in the parameter space)}
}
\value{
rr : variances-covariances matrix on the reference population
}
\description{
calculates the variances-covariances matrix on the reference population
}
\examples{
# Definition of the parameters
x <- matrix(rexp(30), 30, 1)
g <- mean(x)
# Call the function
res <- matvcov(x, g)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vol_splx.R
\name{vol_splx}
\alias{vol_splx}
\title{Volume of a simplex}
\usage{
vol_splx(S)
}
\arguments{
\item{S}{: matrix (d+1) rows * d columns containing the coordinates in d-dim of d + 1 vertices of a simplex}
}
\value{
V : simplex volume
}
\description{
calculates the volume of a simplex
}
\examples{
# Definition of the parameters
S <- matrix(rexp(6), 3, 2)
# Call the function
res <- vol_splx(S)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrease_pop.R
\name{decrease_pop}
\alias{decrease_pop}
\title{Decreasing of the population of parameters sets}
\usage{
decrease_pop(matobj, minmax, prec, archsize, popsize)
}
\arguments{
\item{matobj}{: matrix of objectives, dimension (ngames, nobj)}

\item{minmax}{: vector of booleans, of dimension nobj: TRUE if maximization of the objective, FALSE otherwise}

\item{prec}{: nobj dimension vector: accuracy}

\item{archsize}{: integer: archive size}

\item{popsize}{: integer: population size}
}
\value{
A list containing two elements:
\describe{
\item{ind_arch}{indices of individuals in the updated Pareto front}
\item{ind_pop}{indices of individuals in the updated population}
}
}
\description{
decreases the population of parameters sets
}
\examples{
# Definition of the parameters
matobj <- matrix(rexp(200), 100, 2)
prec <- c(1.e-3, 1.e-3)
archsize <- 100
minmax <- c(FALSE, FALSE)
popsize <- 100
# Call the function
res <- decrease_pop(matobj, minmax, prec, archsize, popsize)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Dimprove.R
\name{Dimprove}
\alias{Dimprove}
\title{Determination of directions for improvement}
\usage{
Dimprove(o_splx, f_splx)
}
\arguments{
\item{o_splx}{: matrix of objectives of simplexes (nrow = npoints, ncol = nobj)}

\item{f_splx}{: vector (npoints) of associated Pareto numbers (1 = dominated)}
}
\value{
list of elements "oriedge": oriented edges and "ledge": length
}
\description{
determines directions for improvement
}
\examples{
# Definition of the parameters
o_splx <- matrix(rexp(6), 3, 2)
f_splx <- c(1,1,1)
# Call the function
res <- Dimprove(o_splx, f_splx)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cinterp.R
\name{Cinterp}
\alias{Cinterp}
\title{Interpolation in simplexes of the objective space}
\usage{
Cinterp(param, crit, simplices, volume, n)
}
\arguments{
\item{param}{: matrix [ NPoints , NPar ] of already evaluated parameters}

\item{crit}{: matrix [ Npoints , NObj ] of associated criteria}

\item{simplices}{: matrix [ NSimp , NObj+1 ] containing all or part of the triangulation of the space of the objectives}

\item{volume}{: matrix [ NSimp , 1 ] giving the volume of each simplex (measure of the probability of interpolating in this simplex)}

\item{n}{: number of new vectors to generate}
}
\value{
xnew : matrix [ n , NPar ] of new vectors

pcrit : matrix [ n , NObj ] estimated positions of new sets in the goal space
}
\description{
proposes n new candidates by interpolation in simplexes of the objective space
}
\examples{
# Definition of the parameters
param <- matrix(rexp(100), 100, 1)
crit <- matrix(rexp(200), 100, 2)
simplices <- matrix(c(15,2,1,15,22,1,18,15,2,17,13,14), nrow = 4, ncol = 3)
volume <- runif(4)
n <- 5
# Call the function
res <- Cinterp(param, crit, simplices, volume, n)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rselect.R
\name{rselect}
\alias{rselect}
\title{Selection of n points}
\usage{
rselect(n, facp)
}
\arguments{
\item{n}{: number of points to select}

\item{facp}{: vector of initial points}
}
\value{
ix : ranks of selected points (vector of dimension n)
}
\description{
performs a selection of n points in facp
}
\examples{
# Definition of the parameters
n <- 5
facp <- runif(30)
# Call the function
res <- rselect(n, facp)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_population.R
\name{plot_population}
\alias{plot_population}
\title{Plotting of a population of objectives}
\usage{
plot_population(
  MatObj,
  nobj,
  ngen = NULL,
  nrun = NULL,
  objnames = NULL,
  MatEvol = NULL,
  popsize = 0
)
}
\arguments{
\item{MatObj}{: matrix of the objectives [NInd, nobj]}

\item{nobj}{: number of objectives}

\item{ngen}{: number of generations (optional)}

\item{nrun}{: number of model evaluations (optional)}

\item{objnames}{: vector of objectives names (optional)}

\item{MatEvol}{: matrix of the evolution of the optimal objectives (optional)}

\item{popsize}{: integer, size of the initial population (optional)}
}
\description{
Plot graphs the population regarding each couple of objectives
}
\examples{
# Definition of the population
Pop <- matrix(runif(300), 100, 3)
# Call the function
plot_population(MatObj = Pop, nobj = 3, objnames = c("Obj1", "Obj2", "Obj3"))

}
\author{
Celine Monteil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pareto.R
\name{plot_pareto}
\alias{plot_pareto}
\title{Plotting of a population of objectives and Pareto front}
\usage{
plot_pareto(MatObj, nobj = NULL, objnames = NULL, maximized = NULL)
}
\arguments{
\item{MatObj}{: matrix of the objectives [NInd, nobj]}

\item{nobj}{: number of objectives (optional)}

\item{objnames}{: vector, length nobj, of names of the objectives (optional)}

\item{maximized}{: vector of logical, length nobj, TRUE if objective need to be maximized, FALSE if minimized}
}
\description{
Plots graphs the population regarding each couple of objectives and emphasizes the Pareto front
}
\examples{
# Definition of the population
Pop <- matrix(runif(300), 100, 3)

# Definition of objectives to maximize (Obj1, Obj2) and to minimize (Obj3)
maximized <- c(TRUE, TRUE, FALSE)

# Call the function
plot_pareto(MatObj = Pop, maximized = maximized)

}
\author{
Celine Monteil
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxes.R
\name{boxes}
\alias{boxes}
\title{Box numbering for each points individual of the population}
\usage{
boxes(points, prec)
}
\arguments{
\item{points}{: matrix of the objectives}

\item{prec}{: (double, length = nobj) desired accuracy for the objectives (edges of the boxes)}
}
\value{
vector of numbers for the boxes. boxes[i] gives the number of the box containing points[i].
}
\description{
This function returns a box number for each points individual of the population
}
\examples{
# Definition of the parameters
points <- matrix(rexp(200), 100, 2)
prec <- c(1.e-3, 1.e-3)
# Call the function
res <- boxes(points, prec)

}
\author{
Fabrice Zaoui
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_caramel.R
\name{plot_caramel}
\alias{plot_caramel}
\title{Plotting of caRamel results}
\usage{
plot_caramel(caramel_results, nobj = NULL, objnames = NULL)
}
\arguments{
\item{caramel_results}{: list resulting from the caRamel() function, with fields $objectives and $save_crit}

\item{nobj}{: number of objectives (optional)}

\item{objnames}{: vector of objectives names (optional)}
}
\description{
Plot graphs of the Pareto front and a graph of optimization evolution
}
\examples{
# Definition of the test function
viennet <- function(i) {
  val1 <- 0.5*(x[i,1]*x[i,1]+x[i,2]*x[i,2])+sin(x[i,1]*x[i,1]+x[i,2]*x[i,2])
  val2 <- 15+(x[i,1]-x[i,2]+1)*(x[i,1]-x[i,2]+1)/27+(3*x[i,1]-2*x[i,2]+4)*(3*x[i,1]-2*x[i,2]+4)/8
  val3 <- 1/(x[i,1]*x[i,1]+x[i,2]*x[i,2]+1) -1.1*exp(-(x[i,1]*x[i,1]+x[i,2]*x[i,2]))
  return(c(val1,val2,val3))
}
nobj <- 3 # Number of objectives
nvar <- 2 # Number of variables
minmax <- c(FALSE, FALSE, FALSE) # All the objectives are to be minimized
bounds <- matrix(data = 1, nrow = nvar, ncol = 2) # Define the bound constraints
bounds[, 1] <- -3 * bounds[, 1]
bounds[, 2] <- 3 * bounds[, 2]

# Caramel optimization
results <- caRamel(nobj, nvar, minmax, bounds, viennet, popsize = 100, archsize = 100,
          maxrun = 500, prec = matrix(1.e-3, nrow = 1, ncol = nobj), carallel = FALSE)

# Plot of results
plot_caramel(results)
}
